#!/usr/bin/env python
#-*- coding: utf-8 -*-
#-*- coding: iso-8859-15 -*-

"""-------------------------------------------------------------------------
             _____ _____ _   _  _____ _    _ _____ _   _  ____             
            |  __ \_   _| \ | |/ ____| |  | |_   _| \ | |/ __ \            
            | |__) || | |  \| | |  __| |  | | | | |  \| | |  | |           
            |  ___/ | | | . ` | | |_ | |  | | | | | . ` | |  | |           
            | |    _| |_| |\  | |__| | |__| |_| |_| |\  | |__| |           
  ____ ___  |_|   |_____|_| \_|\_____|\____/|_____|_| \_|\____/_           
 |___ \__ \      | |   (_) |   | |  | |     | |               | |          
   __) | ) |_____| |__  _| |_  | |  | |_ __ | | ___   __ _  __| | ___ _ __ 
  |__ < / /______| '_ \| | __| | |  | | '_ \| |/ _ \ / _` |/ _` |/ _ \ '__|
  ___) / /_      | |_) | | |_  | |__| | |_) | | (_) | (_| | (_| |  __/ |   
 |____/____|     |_.__/|_|\__|  \____/| .__/|_|\___/ \__,_|\__,_|\___|_|   
                                      | |                                  
                                      |_|                                  
    Author:         Regis Blanchot <rblanchot@gmail.com>
    --------------------------------------------------------------------
    2015-03-27      RB -     
    2016-08-29      RB - added usb.core functions
    --------------------------------------------------------------------
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-------------------------------------------------------------------------"""

# Based on UBW32 / Microchip mphidflash software licensed (GNU GPL v3) by Phillip Burgess <pburgess@dslextreme.com>
# UBW32 sources at : http://vak-opensource.googlecode.com/svn/!svn/bc/117/trunk/utilities/ubw32/
# MPHIDFLASH sources at : http://mphidflash.googlecode.com/svn-history/r2/trunk/
# PyUSB Doc : http://wiki.erazor-zone.de/wiki:projects:python:pyusb:pydoc
# Device Descriptors : lsusb -v -d 04d8:003C

#import sys
import os
import time
import usb
import logging

from .uploader import baseUploader

class uploader32(baseUploader):
    """ upload .hex into pinguino device """

    # Data block description
    BYTESPERADDRESS                 =    1
    MAXPACKETSIZE                   =    64
    DATABLOCKSIZE                   =    56        # MAXPACKETSIZE - Block Command Size

    # Response packet structure
    # ----------------------------------------------------------------------

    """
    struct __attribute__ ((packed))
    {
        unsigned char Command;
        unsigned char PacketDataFieldSize;
        unsigned char DeviceFamily;
        unsigned char Type1;
        unsigned long Address1;
        unsigned long Length1;
        unsigned char Type2;
        unsigned long Address2;
        unsigned long Length2;
        unsigned char Type3;        //End of sections list indicator goes here, when not programming the vectors, in that case fill with 0xFF.
        unsigned long Address3;
        unsigned long Length3;
        unsigned char Type4;        //End of sections list indicator goes here, fill with 0xFF.
        unsigned char ExtraPadBytes[33];
    };
    """

    BOOT_CMD                        =    0      # char = 1 byte
    BOOT_PACKET_SIZE                =    1      # char = 1 byte
    BOOT_DEVICE_FAMILY              =    2      # char = 1 byte
    BOOT_TYPE1                      =    3      # char = 1 byte
    BOOT_ADDR1                      =    4      # long = 4 bytes
    BOOT_IVTSTART                   =    4      # long = 4 bytes
    BOOT_LEN1                       =    8      # long = 4 bytes
    BOOT_MEMFREE                    =    8      # long = 4 bytes
    BOOT_TYPE2                      =    12     # char = 1 byte
    BOOT_ADDR2                      =    13     # long = 4 bytes
    BOOT_LEN2                       =    17     # long = 4 bytes
    BOOT_TYPE3                      =    21     # char = 1 byte
    BOOT_ADDR3                      =    22     # long = 4 bytes
    BOOT_LEN3                       =    26     # long = 4 bytes

    BOOT_TYPE_LEN                   =    9      # char + long + long = 9 bytes

    BOOT_DEVID1                     =    8
    BOOT_DEVID2                     =    60

    BOOT_VER_MAJOR                  =    22
    BOOT_VER_MINOR                  =    26

    # Sent packet structure
    # ----------------------------------------------------------------------

    """
    struct __attribute__ ((packed))
    {
        unsigned char Command;
        unsigned long Address;
        unsigned char Size;
        unsigned char PadBytes[(TotalPacketSize - 6) - (RequestDataBlockSize)];
        unsigned int Data[RequestDataBlockSize/WORDSIZE];
    };
    """

    BOOT_ADDR                       =    1      # 4 bytes
    BOOT_CMD_SIZE                   =    5      # 1 byte
    BOOT_CMD_PAD                    =    6      # 2 bytes unsused
    BOOT_CMD_DATA                   =    8      # 64 - 8 = 56 (DATABLOCKSIZE)

    # Command Definitions
    # ----------------------------------------------------------------------

    UNLOCKCONFIG_CMD                =    0x00    # sub-command for the ERASE_DEVICE_CMD
    LOCKCONFIG_CMD                  =    0x01    # sub-command for the ERASE_DEVICE_CMD
    QUERY_DEVICE_CMD                =    0x02    # what regions can be programmed, and what type of memory is the region
    UNLOCK_CONFIG_CMD               =    0x03    # for both locking and unlocking the config bits
    ERASE_DEVICE_CMD                =    0x04    # to start an erase operation, firmware controls which pages should be erased
    WRITE_DEVICE_CMD                =    0x05    # to send a full RequestDataBlockSize to be programmed
    WRITE_COMPLETE_CMD              =    0x06    # if host send less than a RequestDataBlockSize to be programmed, or if it wished to program whatever was left in the buffer, it uses this command
    READ_DEVICE_CMD                 =    0x07    # the host sends this command in order to read out memory from the device. Used during verify (and read/export hex operations)
    RESET_DEVICE_CMD                =    0x08    # resets the microcontroller, so it can update the config bits (if they were programmed, and so as to leave the bootloader (and potentially go back into the main application)

    READ_DEVICE_CMD_SUPPORTED       =    False
    QUERY_DEVICE_CMD_SUPPORTED      =    False
    READ_DEVICE_BUFFER              =    []
    QUERY_DEVICE_BUFFER             =    []

    # Query Device Response
    # ----------------------------------------------------------------------

    TypeProgramMemory               =    0x01    # when the host sends a QUERY_DEVICE command, need to respond by populating a list of valid memory regions that exist in the device (and should be programmed)
    TypeEEPROM                      =    0x02
    TypeConfigWords                 =    0x03
    TypeEndOfTypeList               =    0xFF    # sort of serves as a "null terminator" like number, which denotes the end of the memory region list has been reached.

    # hid endpoints
    # ----------------------------------------------------------------------
    IN_EP                           =    0x81    # endpoint for Hid reads
    OUT_EP                          =    0x01    # endpoint for Hid writes

    # Device family
    # ----------------------------------------------------------------------
    DEVICE_FAMILY_PIC18             =    0x01
    DEVICE_FAMILY_PIC24             =    0x02
    DEVICE_FAMILY_PIC32             =    0x03
    DEVICE_FAMILY_MX1               =    0x81
    DEVICE_FAMILY_MX3               =    0x82
    DEVICE_FAMILY_MZ                =    0x83

    # Configuration
    # ----------------------------------------------------------------------

    # RB 2015-03-20 : moved to uploader.py
    #INTERFACE_ID                    =    0x00
    #ACTIVE_CONFIG                   =    0x01
    TIMEOUT                         =    10000

    # Memory's area
    # ----------------------------------------------------------------------

    DEVICE_ID_ADDRESS               =    0xBF80F220

    # Table with supported USB devices
    # device_id:['CPU name']
    #-----------------------------------------------------------------------
    """
            0x04A00053: ['32MX220F032B'],
            0x04A04053: ['32MX220F032D'],
            0x04D00053: ['32MX250F128B'],
            0x06600053: ['32MX270F256B'],
            0x00952053: ['32MX440F256H'],
            0x00978053: ['32MX460F512L'],
            0x0430E053: ['32MX795F512H'],
            0x04307053: ['32MX795F512L']
    """

    devices_table = \
        {
            # MX1/2
            0x4A07053:['32MX110F016B',     16,   DEVICE_FAMILY_MX1],
            0x4A09053:['32MX110F016C',     16,   DEVICE_FAMILY_MX1],
            0x4A0B053:['32MX110F016D',     16,   DEVICE_FAMILY_MX1],
            0x4A06053:['32MX120F032B',     32,   DEVICE_FAMILY_MX1],
            0x4A08053:['32MX120F032C',     32,   DEVICE_FAMILY_MX1],
            0x4A0A053:['32MX120F032D',     32,   DEVICE_FAMILY_MX1],
            0x4D07053:['32MX130F064B',     64,   DEVICE_FAMILY_MX1],
            0x4D09053:['32MX130F064C',     64,   DEVICE_FAMILY_MX1],
            0x4D0B053:['32MX130F064D',     64,   DEVICE_FAMILY_MX1],
            0x4D06053:['32MX150F128B',    128,   DEVICE_FAMILY_MX1],
            0x4D08053:['32MX150F128C',    128,   DEVICE_FAMILY_MX1],
            0x4D0A053:['32MX150F128D',    128,   DEVICE_FAMILY_MX1],
            0x4A01053:['32MX210F016B',     16,   DEVICE_FAMILY_MX1],
            0x4A03053:['32MX210F016C',     16,   DEVICE_FAMILY_MX1],
            0x4A05053:['32MX210F016D',     16,   DEVICE_FAMILY_MX1],
            0x4A00053:['32MX220F032B',     32,   DEVICE_FAMILY_MX1],
            0x4A02053:['32MX220F032C',     32,   DEVICE_FAMILY_MX1],
            0x4A04053:['32MX220F032D',     32,   DEVICE_FAMILY_MX1],
            0x4D01053:['32MX230F064B',     64,   DEVICE_FAMILY_MX1],
            0x4D03053:['32MX230F064C',     64,   DEVICE_FAMILY_MX1],
            0x4D05053:['32MX230F064D',     64,   DEVICE_FAMILY_MX1],
            0x4D00053:['32MX250F128B',    128,   DEVICE_FAMILY_MX1],
            0x4D02053:['32MX250F128C',    128,   DEVICE_FAMILY_MX1],
            0x4D04053:['32MX250F128D',    128,   DEVICE_FAMILY_MX1],
            0x6600053:['32MX270F256B',    256,   DEVICE_FAMILY_MX1],
            0x6602053:['32MX270F256C',    256,   DEVICE_FAMILY_MX1],
            0x6604053:['32MX270F256D',    256,   DEVICE_FAMILY_MX1],
            # MX3/4/5/6/7
            0x0902053:['32MX320F032H',     32,   DEVICE_FAMILY_MX3],
            0x0906053:['32MX320F064H',     64,   DEVICE_FAMILY_MX3],
            0x090A053:['32MX320F128H',    128,   DEVICE_FAMILY_MX3],
            0x092A053:['32MX320F128L',    128,   DEVICE_FAMILY_MX3],
            0x5600053:['32MX330F064H',     64,   DEVICE_FAMILY_MX3],
            0x5601053:['32MX330F064L',     64,   DEVICE_FAMILY_MX3],
            0x090D053:['32MX340F128H',    128,   DEVICE_FAMILY_MX3],
            0x092D053:['32MX340F128L',    128,   DEVICE_FAMILY_MX3],
            0x0912053:['32MX340F256H',    256,   DEVICE_FAMILY_MX3],
            0x0916053:['32MX340F512H',    512,   DEVICE_FAMILY_MX3],
            0x570C053:['32MX350F128H',    128,   DEVICE_FAMILY_MX3],
            0x570D053:['32MX350F128L',    128,   DEVICE_FAMILY_MX3],
            0x5704053:['32MX350F256H',    256,   DEVICE_FAMILY_MX3],
            0x5705053:['32MX350F256L',    256,   DEVICE_FAMILY_MX3],
            0x0934053:['32MX360F256L',    256,   DEVICE_FAMILY_MX3],
            0x0938053:['32MX360F512L',    512,   DEVICE_FAMILY_MX3],
            0x5808053:['32MX370F512H',    512,   DEVICE_FAMILY_MX3],
            0x5809053:['32MX370F512L',    512,   DEVICE_FAMILY_MX3],
            0x0942053:['32MX420F032H',     32,   DEVICE_FAMILY_MX3],
            0x5602053:['32MX430F064H',     64,   DEVICE_FAMILY_MX3],
            0x5603053:['32MX430F064L',     64,   DEVICE_FAMILY_MX3],
            0x094D053:['32MX440F128H',    128,   DEVICE_FAMILY_MX3],
            0x096D053:['32MX440F128L',    128,   DEVICE_FAMILY_MX3],
            0x0952053:['32MX440F256H',    256,   DEVICE_FAMILY_MX3],
            0x0956053:['32MX440F512H',    512,   DEVICE_FAMILY_MX3],
            0x570E053:['32MX450F128H',    128,   DEVICE_FAMILY_MX3],
            0x570F053:['32MX450F128L',    128,   DEVICE_FAMILY_MX3],
            0x5706053:['32MX450F256H',    256,   DEVICE_FAMILY_MX3],
            0x5707053:['32MX450F256L',    256,   DEVICE_FAMILY_MX3],
            0x0974053:['32MX460F256L',    256,   DEVICE_FAMILY_MX3],
            0x0978053:['32MX460F512L',    512,   DEVICE_FAMILY_MX3],
            0x580A053:['32MX470F512H',    512,   DEVICE_FAMILY_MX3],
            0x580B053:['32MX470F512L',    512,   DEVICE_FAMILY_MX3],
            0x4400053:['32MX534F064H',     64,   DEVICE_FAMILY_MX3],
            0x440C053:['32MX534F064L',     64,   DEVICE_FAMILY_MX3],
            0x4401053:['32MX564F064H',     64,   DEVICE_FAMILY_MX3],
            0x440D053:['32MX564F064L',     64,   DEVICE_FAMILY_MX3],
            0x4403053:['32MX564F128H',    128,   DEVICE_FAMILY_MX3],
            0x440F053:['32MX564F128L',    128,   DEVICE_FAMILY_MX3],
            0x4317053:['32MX575F256H',    256,   DEVICE_FAMILY_MX3],
            0x4333053:['32MX575F256L',    256,   DEVICE_FAMILY_MX3],
            0x4309053:['32MX575F512H',    512,   DEVICE_FAMILY_MX3],
            0x430F053:['32MX575F512L',    512,   DEVICE_FAMILY_MX3],
            0x4405053:['32MX664F064H',     64,   DEVICE_FAMILY_MX3],
            0x4411053:['32MX664F064L',     64,   DEVICE_FAMILY_MX3],
            0x4407053:['32MX664F128H',    128,   DEVICE_FAMILY_MX3],
            0x4413053:['32MX664F128L',    128,   DEVICE_FAMILY_MX3],
            0x430B053:['32MX675F256H',    256,   DEVICE_FAMILY_MX3],
            0x4305053:['32MX675F256L',    256,   DEVICE_FAMILY_MX3],
            0x430C053:['32MX675F512H',    512,   DEVICE_FAMILY_MX3],
            0x4311053:['32MX675F512L',    512,   DEVICE_FAMILY_MX3],
            0x4325053:['32MX695F512H',    512,   DEVICE_FAMILY_MX3],
            0x4341053:['32MX695F512L',    512,   DEVICE_FAMILY_MX3],
            0x440B053:['32MX764F128H',    128,   DEVICE_FAMILY_MX3],
            0x4417053:['32MX764F128L',    128,   DEVICE_FAMILY_MX3],
            0x4303053:['32MX775F256H',    256,   DEVICE_FAMILY_MX3],
            0x4312053:['32MX775F256L',    256,   DEVICE_FAMILY_MX3],
            0x430D053:['32MX775F512H',    512,   DEVICE_FAMILY_MX3],
            0x4306053:['32MX775F512L',    512,   DEVICE_FAMILY_MX3],
            0x430E053:['32MX795F512H',    512,   DEVICE_FAMILY_MX3],
            0x4307053:['32MX795F512L',    512,   DEVICE_FAMILY_MX3],
            # MZ
            0x5100053:['32MZ0256ECE064',  256,   DEVICE_FAMILY_MZ],
            0x510A053:['32MZ0256ECE100',  256,   DEVICE_FAMILY_MZ],
            0x5114053:['32MZ0256ECE124',  256,   DEVICE_FAMILY_MZ],
            0x511E053:['32MZ0256ECE144',  256,   DEVICE_FAMILY_MZ],
            0x5105053:['32MZ0256ECF064',  256,   DEVICE_FAMILY_MZ],
            0x510F053:['32MZ0256ECF100',  256,   DEVICE_FAMILY_MZ],
            0x5119053:['32MZ0256ECF124',  256,   DEVICE_FAMILY_MZ],
            0x5123053:['32MZ0256ECF144',  256,   DEVICE_FAMILY_MZ],
            0x5101053:['32MZ0512ECE064',  512,   DEVICE_FAMILY_MZ],
            0x510B053:['32MZ0512ECE100',  512,   DEVICE_FAMILY_MZ],
            0x5115053:['32MZ0512ECE124',  512,   DEVICE_FAMILY_MZ],
            0x511F053:['32MZ0512ECE144',  512,   DEVICE_FAMILY_MZ],
            0x5106053:['32MZ0512ECF064',  512,   DEVICE_FAMILY_MZ],
            0x5110053:['32MZ0512ECF100',  512,   DEVICE_FAMILY_MZ],
            0x511A053:['32MZ0512ECF124',  512,   DEVICE_FAMILY_MZ],
            0x5124053:['32MZ0512ECF144',  512,   DEVICE_FAMILY_MZ],
            0x5102053:['32MZ1024ECE064', 1024,   DEVICE_FAMILY_MZ],
            0x510C053:['32MZ1024ECE100', 1024,   DEVICE_FAMILY_MZ],
            0x5116053:['32MZ1024ECE124', 1024,   DEVICE_FAMILY_MZ],
            0x5120053:['32MZ1024ECE144', 1024,   DEVICE_FAMILY_MZ],
            0x5107053:['32MZ1024ECF064', 1024,   DEVICE_FAMILY_MZ],
            0x5111053:['32MZ1024ECF100', 1024,   DEVICE_FAMILY_MZ],
            0x511B053:['32MZ1024ECF124', 1024,   DEVICE_FAMILY_MZ],
            0x5125053:['32MZ1024ECF144', 1024,   DEVICE_FAMILY_MZ],
            0x5103053:['32MZ1024ECG064', 1024,   DEVICE_FAMILY_MZ],
            0x510D053:['32MZ1024ECG100', 1024,   DEVICE_FAMILY_MZ],
            0x5117053:['32MZ1024ECG124', 1024,   DEVICE_FAMILY_MZ],
            0x5121053:['32MZ1024ECG144', 1024,   DEVICE_FAMILY_MZ],
            0x5108053:['32MZ1024ECH064', 1024,   DEVICE_FAMILY_MZ],
            0x5112053:['32MZ1024ECH100', 1024,   DEVICE_FAMILY_MZ],
            0x511C053:['32MZ1024ECH124', 1024,   DEVICE_FAMILY_MZ],
            0x5126053:['32MZ1024ECH144', 1024,   DEVICE_FAMILY_MZ],
            0x5130053:['32MZ1024ECM064', 1024,   DEVICE_FAMILY_MZ],
            0x513A053:['32MZ1024ECM100', 1024,   DEVICE_FAMILY_MZ],
            0x5144053:['32MZ1024ECM124', 1024,   DEVICE_FAMILY_MZ],
            0x514E053:['32MZ1024ECM144', 1024,   DEVICE_FAMILY_MZ],
            0x5104053:['32MZ2048ECG064', 2048,   DEVICE_FAMILY_MZ],
            0x510E053:['32MZ2048ECG100', 2048,   DEVICE_FAMILY_MZ],
            0x5118053:['32MZ2048ECG124', 2048,   DEVICE_FAMILY_MZ],
            0x5122053:['32MZ2048ECG144', 2048,   DEVICE_FAMILY_MZ],
            0x5109053:['32MZ2048ECH064', 2048,   DEVICE_FAMILY_MZ],
            0x5113053:['32MZ2048ECH100', 2048,   DEVICE_FAMILY_MZ],
            0x511D053:['32MZ2048ECH124', 2048,   DEVICE_FAMILY_MZ],
            0x5127053:['32MZ2048ECH144', 2048,   DEVICE_FAMILY_MZ],
            0x5131053:['32MZ2048ECM064', 2048,   DEVICE_FAMILY_MZ],
            0x513B053:['32MZ2048ECM100', 2048,   DEVICE_FAMILY_MZ],
            0x5145053:['32MZ2048ECM124', 2048,   DEVICE_FAMILY_MZ],
            0x514F053:['32MZ2048ECM144', 2048,   DEVICE_FAMILY_MZ]
        }

# ----------------------------------------------------------------------
    def resetDevice(self, handle):
# ----------------------------------------------------------------------
        """ Reset device """
        usbBuf = [self.RESET_DEVICE_CMD] * self.MAXPACKETSIZE

        try:
            if self.PYUSB_USE_CORE:
                handle.write(self.OUT_EP, usbBuf, self.TIMEOUT)
            else:
                handle.interruptWrite(self.OUT_EP, usbBuf, self.TIMEOUT)
                
        except Exception as e:
            logging.info(e)
            return self.ERR_USB_WRITE

        return self.ERR_NONE

# ----------------------------------------------------------------------
    def sendPacket(self, handle, usbBuf):
# ----------------------------------------------------------------------
        """ Send a packet to the bootloader """

        try:
            if self.PYUSB_USE_CORE:
                sent_bytes = handle.write(self.OUT_EP, usbBuf, self.TIMEOUT)
            else:
                sent_bytes = handle.interruptWrite(self.OUT_EP, usbBuf, self.TIMEOUT)

        except Exception as e:
            logging.info(e)
            return self.ERR_USB_WRITE

        if sent_bytes == len(usbBuf):
            #logging.info("%d bytes successfully sent." % sent_bytes)
            return self.ERR_NONE

        else:
            logging.info("Sent %d/%d bytes" % (sent_bytes, len(usbBuf)))
            return self.ERR_USB_WRITE

# ----------------------------------------------------------------------
    def sendCommand(self, handle, command):
# ----------------------------------------------------------------------
        """ Send a command to the bootloader """

        usbBuf = [command] * self.MAXPACKETSIZE
        return self.sendPacket(handle, usbBuf)

# ----------------------------------------------------------------------
    def getResponse(self, handle):
# ----------------------------------------------------------------------
        """ Get a response from the bootloader """
        """ Assumes a command has been sent before """

        try:
            if self.PYUSB_USE_CORE:
                usbBuf = handle.read(self.IN_EP, self.MAXPACKETSIZE, self.TIMEOUT)
            else:
                usbBuf = handle.interruptRead(self.IN_EP, self.MAXPACKETSIZE, self.TIMEOUT)

        except Exception as e:
            logging.info("Received nothing : %s" % e)
            return self.ERR_USB_READ

        logging.info("Received a packet")
        """
        logging.info("Received : %s" % str(usbBuf).strip('[]'))
        for i in range(sent_bytes):
            logging.info("usbBuf[%d] = %02X" % (i, usbBuf[i]) )
        """

        return usbBuf

# ----------------------------------------------------------------------
    def getCommands(self, handle, board):
# ----------------------------------------------------------------------
        """ Get supported commands """

        # 1/ try QUERY_DEVICE_CMD

        logging.info("Trying with QUERY_DEVICE ...")
        status = self.sendCommand(handle, self.QUERY_DEVICE_CMD)
        if status == self.ERR_NONE:
            self.QUERY_DEVICE_BUFFER = self.getResponse(handle)
            if self.QUERY_DEVICE_BUFFER == self.ERR_USB_READ:
                logging.info("QUERY_DEVICE not supported")
                self.QUERY_DEVICE_CMD_SUPPORTED = False
            else:
                logging.info("QUERY_DEVICE supported")
                self.QUERY_DEVICE_CMD_SUPPORTED = True
        else :
            return status

        # 2/ try GET_DATA_CMD

        if board.proc != '32MX220F032D' and \
           board.proc != '32MX220F032B' and \
           board.proc != '32MX250F128B':

            logging.info("Trying with GET_DATA ...")
            usbBuf = [self.READ_DEVICE_CMD] * self.MAXPACKETSIZE
            usbBuf[self.BOOT_ADDR + 0] = (self.DEVICE_ID_ADDRESS      ) & 0xFF
            usbBuf[self.BOOT_ADDR + 1] = (self.DEVICE_ID_ADDRESS >> 8 ) & 0xFF
            usbBuf[self.BOOT_ADDR + 2] = (self.DEVICE_ID_ADDRESS >> 16) & 0xFF
            usbBuf[self.BOOT_ADDR + 3] = (self.DEVICE_ID_ADDRESS >> 24) & 0xFF
            usbBuf[self.BOOT_CMD_SIZE] = 4

            status = self.sendPacket(handle, usbBuf)
            if status == self.ERR_NONE:
                self.READ_DEVICE_BUFFER = self.getResponse(handle)
                if self.READ_DEVICE_BUFFER == self.ERR_USB_READ:
                    logging.info("GET_DATA not supported")
                    self.READ_DEVICE_CMD_SUPPORTED = False
                else:
                    logging.info("GET_DATA supported")
                    self.READ_DEVICE_CMD_SUPPORTED = True
            else :
                return status

        return self.ERR_NONE

# ----------------------------------------------------------------------
    def getDeviceFamily(self):
# ----------------------------------------------------------------------
        """ Get device info """
        if self.QUERY_DEVICE_CMD_SUPPORTED == True:
            return self.QUERY_DEVICE_BUFFER[self.BOOT_DEVICE_FAMILY]
        else:
            return False

# ----------------------------------------------------------------------
    def getDeviceFCPU(self):
# ----------------------------------------------------------------------
        """ Get device info """
        if self.QUERY_DEVICE_CMD_SUPPORTED == True:
            # CPU frequency
            fcpu = (self.QUERY_DEVICE_BUFFER[self.BOOT_ADDR2 + 0]      ) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_ADDR2 + 1] <<  8) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_ADDR2 + 2] << 16) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_ADDR2 + 3] << 24)
            return fcpu
        else:
            return False

# ----------------------------------------------------------------------
    def getDeviceFPB(self):
# ----------------------------------------------------------------------
        """ Get device info """
        if self.QUERY_DEVICE_CMD_SUPPORTED == True:
            # Peripheral frequency
            fpb  = (self.QUERY_DEVICE_BUFFER[self.BOOT_LEN2 + 0]      ) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_LEN2 + 1] <<  8) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_LEN2 + 2] << 16) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_LEN2 + 3] << 24)
            return fpb
        else:
            return False

# ----------------------------------------------------------------------
    def getDeviceID(self):
# ----------------------------------------------------------------------
        """ Get 4-byte device ID """
        if self.READ_DEVICE_CMD_SUPPORTED == True:
            devid1 = (self.READ_DEVICE_BUFFER[self.BOOT_DEVID1 + 0]      ) | \
                (self.READ_DEVICE_BUFFER[self.BOOT_DEVID1 + 1] <<  8) | \
                (self.READ_DEVICE_BUFFER[self.BOOT_DEVID1 + 2] << 16) | \
                (self.READ_DEVICE_BUFFER[self.BOOT_DEVID1 + 3] << 24)
            devid2 = (self.READ_DEVICE_BUFFER[self.BOOT_DEVID2 + 0]      ) | \
                (self.READ_DEVICE_BUFFER[self.BOOT_DEVID2 + 1] <<  8) | \
                (self.READ_DEVICE_BUFFER[self.BOOT_DEVID2 + 2] << 16) | \
                (self.READ_DEVICE_BUFFER[self.BOOT_DEVID2 + 3] << 24)
            # at least one is null
            devid = devid1 + devid2
            return devid
        else:
            return False

# ----------------------------------------------------------------------
    def getVersion(self):
# ----------------------------------------------------------------------
        """ get bootloader version """
        if self.QUERY_DEVICE_CMD_SUPPORTED == True:
            major = self.QUERY_DEVICE_BUFFER[self.BOOT_VER_MAJOR]
            minor = self.QUERY_DEVICE_BUFFER[self.BOOT_VER_MINOR]
            if major == 0 and minor == 0:
                logging.info("No")
                return False
            else:
                logging.info("Yes, major = %d" % major)
                logging.info("Yes, minor = %d" % minor)
                return str(major) + "." + str(minor)
        else:
            return False

# ----------------------------------------------------------------------
    def getDeviceFlashStart(self):
# ----------------------------------------------------------------------
        """ Get user program start address """
        if self.QUERY_DEVICE_CMD_SUPPORTED == True:
            start = (self.QUERY_DEVICE_BUFFER[self.BOOT_IVTSTART + 0]      ) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_IVTSTART + 1] <<  8) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_IVTSTART + 2] << 16) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_IVTSTART + 3] << 24)
            return start
        else:
            return False

# ----------------------------------------------------------------------
    def getDeviceFlashFree(self):
# ----------------------------------------------------------------------
        """ Get available free memory """
        if self.QUERY_DEVICE_CMD_SUPPORTED == True:
            free = (self.QUERY_DEVICE_BUFFER[self.BOOT_MEMFREE + 0]      ) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_MEMFREE + 1] <<  8) | \
                (self.QUERY_DEVICE_BUFFER[self.BOOT_MEMFREE + 2] << 16)
            return free
        else:
            return False

# ----------------------------------------------------------------------
    def getDeviceName(self, device_id):
# ----------------------------------------------------------------------
        """ Get device name from device ID """
        for n in self.devices_table:
            if n == device_id:
                return self.devices_table[n][0]
        return self.ERR_DEVICE_NOT_FOUND

# ----------------------------------------------------------------------
    def readFlash(self, handle, address, length):
# ----------------------------------------------------------------------
        """ read a block of bytes in flash memory"""

        # command code
        usbBuf = [self.READ_DEVICE_CMD] * self.MAXPACKETSIZE

        # address
        usbBuf[self.BOOT_ADDR + 0] = (address      ) & 0xFF
        usbBuf[self.BOOT_ADDR + 1] = (address >> 8 ) & 0xFF
        usbBuf[self.BOOT_ADDR + 2] = (address >> 16) & 0xFF
        usbBuf[self.BOOT_ADDR + 3] = (address >> 24) & 0xFF

        # size of block to read
        usbBuf[self.BOOT_CMD_SIZE] = length

        # send request to the bootloader and return response
        return self.sendPacketWithResponse(usbBuf)

# ----------------------------------------------------------------------
    def eraseFlash(self, handle):
# ----------------------------------------------------------------------
        """ Erase the whole flash memory """
        return self.sendCommand(handle, self.ERASE_DEVICE_CMD)

# ----------------------------------------------------------------------
    def writeFlash(self, handle, address, block):
# ----------------------------------------------------------------------
        """ Write a block of code """

        # Convert Virtual Address to Physical Address if necessary
        # as NVMADDR only accept physical address
        #address = address & 0x1FFFFFFF

        length = len(block)

        #logging.info("writing %d-byte block at 0x%08X" % (length, address))

        #logging.info(str(block).strip('[]'))

        if length == 0:
            logging.info("Zero-length data packet at 0x%08X flushed" % address)
            return self.sendCommand(handle, self.WRITE_COMPLETE_CMD)

        """
        struct __attribute__ ((packed))
        {
            unsigned char Command;
            unsigned long Address;
            unsigned char Size;
            unsigned char PadBytes[(TotalPacketSize - 6) - (RequestDataBlockSize)];
            unsigned int Data[RequestDataBlockSize/WORDSIZE];
        };
        """

        # command code
        usbBuf = [self.WRITE_DEVICE_CMD] * self.MAXPACKETSIZE

        # block start address
        usbBuf[self.BOOT_ADDR + 0] = (address      ) & 0xFF
        usbBuf[self.BOOT_ADDR + 1] = (address >> 8 ) & 0xFF
        usbBuf[self.BOOT_ADDR + 2] = (address >> 16) & 0xFF
        usbBuf[self.BOOT_ADDR + 3] = (address >> 24) & 0xFF

        # data's length in bytes
        usbBuf[self.BOOT_CMD_SIZE] = length

        # pad bytes
        # 64 bytes (USB Packet Size ) - 6 bytes (Command) = 58 not divisible by 4
        # 58 bytes - 2 bytes = 56 bytes (Data block size) which is divisible by 4
        usbBuf[self.BOOT_CMD_PAD + 0] = 0
        usbBuf[self.BOOT_CMD_PAD + 1] = 0

        # add data 'right justified' within packet
        # note : the bootloader will 'left justified' all data at reception
        for i in range(length):
            usbBuf[self.MAXPACKETSIZE - length + i] = block[i]

        #logging.info(str(usbBuf).strip('[]'))

        # write data packet on usb device
        status = self.sendPacket(handle, usbBuf)
        #logging.info("status = %d" % status)

        if (status == self.ERR_NONE) and (length < self.DATABLOCKSIZE):
            # Short data packets need flushing
            logging.info("Short data packet at 0x%08X flushed" % address)
            #usbBuf = [self.WRITE_COMPLETE_CMD] * self.MAXPACKETSIZE
            #usbBuf[self.BOOT_CMD] = self.WRITE_COMPLETE_CMD
            status = self.sendCommand(handle, self.WRITE_COMPLETE_CMD)

        return status

# ----------------------------------------------------------------------
    def writeHex(self, handle, filename, board):
# ----------------------------------------------------------------------
        """
            Parse the Hex File Format and send data to usb device
            [0]      Start code, one character, an ASCII colon ':'.
            [1:3]    Byte count, two hex digits, a number of bytes (hex digit pairs) in the data field. 16 (0x10) or 32 (0x20) bytes of data are the usual compromise values between line length and address overhead.
            [3:7]    Address, four hex digits, a 16-bit address of the beginning of the memory position for the data. Limited to 64 kilobytes, the limit is worked around by specifying higher bits via additional record types. This address is big endian.
            [7:9]    Record type, two hex digits, 00 to 05, defining the type of the data field.
            [9:*]    Data, a sequence of n bytes of the data themselves, represented by 2n hex digits.
            [*:*]    Checksum, two hex digits - the least significant byte of the two's complement of the sum of the values of all fields except fields 1 and 6 (Start code ":" byte and two hex digits of the Checksum). It is calculated by adding together the hex-encoded bytes (hex digit pairs), then leaving only the least significant byte of the result, and making a 2's complement (either by subtracting the byte from 0x100, or inverting it by XOR-ing with 0xFF and adding 0x01). If you are not working with 8-bit variables, you must suppress the overflow by AND-ing the result with 0xFF. The overflow may occur since both 0x100-0 and'], 0x00 XOR 0xFF)+1 equal 0x100. If the checksum is correctly calculated, adding all the bytes (the Byte count, both bytes in Address, the Record type, each Data byte and the Checksum) together will always result in a value wherein the least significant byte is zero'], 0x00).
                     For example, on :0300300002337A1E
                     03 + 00 + 30 + 00 + 02 + 33 + 7A = E2, 2's complement is 1E
        """

        program_memory = []
        last_address = 0
        max_address  = 0
        min_address  = 0xFFFFFFFF
        address_Hi   = 0
        codesize     = 0

        # Memory image (IVT + RESET VECTOR + STARTUP SEQUENCE + USER APP.)
        # --------------------------------------------------------------

        logging.info("writing from min 0x%08X to max 0x%08X ..." % (board.ivtstart,board.memend))
        for i in range(board.memend - board.ivtstart):
            program_memory.append(0xFF)

        # load hex file
        # ----------------------------------------------------------------------

        hexfile = open(filename, 'r')
        lines = hexfile.readlines()
        hexfile.close()

        # read each line in file
        # --------------------------------------------------------------

        for line in lines:

            byte_count = int(line[1:3], 16)
            # lower 16 bits (bits 0-15) of the data address
            address_Lo = int(line[3:7], 16)
            record_type= int(line[7:9], 16)

            # checksum calculation
            # ----------------------------------------------------------

            end = 9 + byte_count * 2 # position of checksum at end of line
            checksum = int(line[end:end+2], 16)

            cs = 0
            i = 1
            while i < end:
                cs = cs + (0x100 - int(line[i:i+2], 16) ) & 0xff # eq. to not(i)
                i = i + 2

            if checksum != cs:
                return self.ERR_HEX_CHECKSUM

            # extended linear address record
            # ----------------------------------------------------------

            if record_type == self.Extended_Linear_Address_Record:

                # upper 16 bits (bits 16-31) of the data address
                address_Hi = int(line[9:13], 16) << 16

            # data record
            # ivtstart = where IVT starts (eBase)
            # memstart = where user's program starts
            # ----------------------------------------------------------

            elif record_type == self.Data_Record:

                address = address_Hi + address_Lo
                #logging.info("address = 0x%08X" % address)

                if (address >= board.ivtstart) and (address < board.memend):

                    # min program address
                    if (min_address > address):
                        min_address = address

                    # max program address
                    if (address > last_address):
                        max_address = address + byte_count
                        last_address = address
                        #logging.info("max_address = 0x%08X" % max_address)

                    # user's program code size
                    if (address >= board.memstart):
                        codesize = codesize + byte_count
                        #logging.info("codesize = %d" % codesize)

                    # program data append
                    for i in range(byte_count):
                        program_memory[address - board.ivtstart + i] = \
                            int(line[9 + (2 * i) : 11 + (2 * i)], 16)

            # Reset Vector
            # ----------------------------------------------------------

            elif record_type == self.Start_Linear_Address_Record:

                logging.info("Reset Vector = 0x%08X" % int(line[9:17], 16))

            # end of file record
            # ----------------------------------------------------------

            elif record_type == self.End_Of_File_Record:

                break

            # unsupported record type
            # ----------------------------------------------------------

            else:

                self.add_report("Caution : unsupported record type (%d) in hex file" % record_type)
                self.add_report("Line %s" % line)
                #return self.ERR_HEX_RECORD

        # min and max address must be divisible by self.DATABLOCKSIZE
        # --------------------------------------------------------------

        # 13/10/2014 - fixed by André
        #max_address = max_address + self.DATABLOCKSIZE - (codesize % self.DATABLOCKSIZE)
        #max_address = max_address + self.DATABLOCKSIZE - (max_address % self.DATABLOCKSIZE)
        #logging.info("program max. address = 0x%08X" % max_address)

        # trim the memory image
        # --------------------------------------------------------------
        """
        logging.info("first byte to write at 0x%08X" % min_address)
        logging.info("last  byte to write at 0x%08X" % max_address)
        logging.info("index min. = %d (0x%X)" % \
            ((min_address - board.ivtstart),(min_address - board.ivtstart)))
        del program_memory[:(min_address - board.ivtstart)]
        logging.info("index max. = %d (0x%X)" % \
            ((max_address - board.ivtstart),(max_address - board.ivtstart)))
        del program_memory[(max_address - board.ivtstart):]
        """
        # write blocks of DATABLOCKSIZE bytes in program memory
        # --------------------------------------------------------------

        min_address = board.ivtstart;

        logging.info("writing from 0x%08X to 0x%08X ..." % (min_address,max_address))
        for addr in range(min_address, max_address, self.DATABLOCKSIZE):
            index = addr - min_address
            #logging.info("address, index = 0x%08X, %d" % (addr, index))
            status = self.writeFlash(handle, addr, program_memory[index:index+self.DATABLOCKSIZE])
            if (status != self.ERR_NONE):
                return status

        # end
        # --------------------------------------------------------------

        logging.info("Writing completed")
        self.add_report("%d bytes written." % codesize)
        status = self.sendCommand(handle, self.WRITE_COMPLETE_CMD)

        return status

# ----------------------------------------------------------------------
    def uploadDevice(self, filename, board):
# ----------------------------------------------------------------------

        # check file to upload
        # --------------------------------------------------------------

        if filename == '':
            self.add_report("No program to write")
            return

        hexfile = open(filename, 'r')

        if hexfile == "":
            self.add_report("Unable to open %s" % filename)
            return

        hexfile.close()

        # search for a Pinguino board
        # --------------------------------------------------------------

        device = self.getDevice(board)

        if device == self.ERR_DEVICE_NOT_FOUND:

            self.add_report("Pinguino not found")
            self.add_report("If your device is connected switch to bootloader mode.")
            return

        self.add_report("Pinguino found ...")

        handle = self.initDevice(device)

        if handle == self.ERR_USB_INIT1:

            self.add_report("Upload not possible")
            self.add_report("Try to restart the bootloader mode")
            return

        # find out supported commands
        # --------------------------------------------------------------

        logging.info("Checking supported commands ...")
        status = self.getCommands(handle, board)

        if status == self.ERR_USB_WRITE:
            self.add_report("Aborting : write error!")
            self.closeDevice(handle)
            return

        """
        if not self.READ_DEVICE_CMD_SUPPORTED or \
           not self.QUERY_DEVICE_CMD_SUPPORTED:
            logging.info("Reseting ...")
            self.closeDevice(handle)
            device = self.getDevice(board)
            handle = self.initDevice(device)
            #logging.info("Device reset")
            #self.resetDevice(handle)
            #handle.reset()
            #time.sleep(5)
            #handle.releaseInterface()
            #time.sleep(5)
            #handle.claimInterface(self.INTERFACE_ID)
            #handle.clearHalt(self.IN_EP)
            #handle.resetEndpoint(self.IN_EP)
            #self.initDevice(device)
        """

        # find out processor family
        # --------------------------------------------------------------

        logging.info("Getting processor family ...")
        fam = self.getDeviceFamily()
        if fam:
            logging.info("Yes, Device family = %d" % fam)
            if  fam != self.DEVICE_FAMILY_PIC32:
                self.add_report("Aborting : not a PIC32 family device.")
                self.closeDevice(handle)
                return
        else:
            logging.info("No")

        # find out processor name
        # --------------------------------------------------------------

        logging.info("Getting processor name ...")
        devid = self.getDeviceID()
        if devid:
            logging.info("Yes, Device ID = %08X" % devid)
            # mask device id to get revision number
            rev = ( ( devid >> 24 ) & 0xF0 ) >> 4
            # mask revision number to get device id
            pid  = devid & 0x0FFFFFFF
            # get proc. name
            proc = self.getDeviceName(pid)
            self.add_report(" - with PIC%s (ID 0x%08X, rev. A%01X)" % (proc, pid, rev))
            if proc != board.proc:
                self.add_report("Aborting: program compiled for %s but device has %s" % (board.proc, proc))
                self.closeDevice(handle)
                return
        else:
            logging.info("No, assuming it's a PIC%s" % board.proc)
            self.add_report(" - with supposed PIC%s" % board.proc)

        # find out clock frequencies
        # --------------------------------------------------------------

        logging.info("Getting clock frequencies ...")
        fcpu = self.getDeviceFCPU()
        fpb = self.getDeviceFPB()
        if fcpu and fpb:
            logging.info("Yes, FCPU = %dHz" % fcpu)
            logging.info("Yes, FPB  = %dHz" % fpb)
            self.add_report("   System  clock %.3f MHz" % ( fcpu/1000000.0 ))
            self.add_report("   Periph. clock %.3f MHz" % (  fpb/1000000.0 ))
        else:
            logging.info("No")

        # find out flash memory size
        # --------------------------------------------------------------
        logging.info("Getting flash memory infos ...")
        memstart = self.getDeviceFlashStart()
        memfree  = self.getDeviceFlashFree()
        if memstart and memfree:
            logging.info("Yes, Prog. starts at 0x%08X" % memstart)
            #logging.info("Yes, free  = 0x%08X" % memfree)
            # Convert KSEG1 to KSEG0
            # and Physical to Virtual address
            if memstart >= 0xBD000000:
                memstart = memstart - 0x20000000
            memstart = memstart | 0x80000000
            ivtstart = board.ivtstart
            memend   = memstart + memfree
            #memfree  = memend - memstart

            if board.memstart != memstart:
                logging.info("Conflict : memstart should be 0x%08X not 0x%08X" % (memstart, board.memstart))
                logging.info("This issue has been automatically fixed.")
                logging.info("Please report it at https://github.com/PinguinoIDE/pinguino-ide/issues")
                board.memstart = memstart

            if board.memend != memend:
                logging.info("Conflict : memend should be 0x%08X not 0x%08X" % (memend, board.memend))
                logging.info("This issue has been automatically fixed.")
                logging.info("Please report it at https://github.com/PinguinoIDE/pinguino-ide/issues")
                board.memend = memend

            logging.info("Yes, ivtstart = 0x%08X" % ivtstart)
            logging.info("Yes, memstart = 0x%08X" % memstart)
            logging.info("Yes, memend   = 0x%08X" % memend)
            logging.info("Yes, memfree  = 0x%X bytes free" % memfree)

        else:
            logging.info("No")
            memfree = board.memend - board.memstart

        self.add_report(" - with %d bytes free (%d KB)" % (memfree, memfree/1024))
        self.add_report("   from 0x%08X to 0x%08X" % (board.memstart, board.memend))

        # find out bootloader version
        # --------------------------------------------------------------

        logging.info("Getting bootloader version ...")
        version = self.getVersion()
        if version:
            self.manufacturer = "Pinguino"
            self.add_report(" - with Pinguino USB HID Bootloader v%s" % version)
        else:
            self.manufacturer = "Microchip"
            self.add_report(" - with Microchip USB HID Bootloader")

        # start erasing
        # --------------------------------------------------------------

        self.add_report("Erasing flash memory ...")
        status = self.eraseFlash(handle)
        if status != self.ERR_NONE:
            self.add_report("Aborting: erase error!")
            self.closeDevice(handle)
            return

        # start writing
        # --------------------------------------------------------------

        self.add_report("Uploading user program ...")
        status = self.writeHex(handle, filename, board)
        if status != self.ERR_NONE:
            self.add_report("Aborting: write error code %d" % status)
            self.closeDevice(handle)
            return

        self.add_report("%s successfully uploaded." % os.path.basename(filename))

        # reset and start user's app.
        # --------------------------------------------------------------

        logging.info("Sending Reset command ...")
        self.add_report("Starting user program ...")
        status = self.resetDevice(handle)
        logging.info("Device Reset failed ...")
        if status != self.ERR_NONE:
            self.closeDevice(handle)

        return

# ----------------------------------------------------------------------



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import logging

from PySide import QtGui

from ..custom_widgets import PinguinoTextEdit


########################################################################
class Log(object):

    def __init__(self):

        self.main.plainTextEdit_log = PinguinoTextEdit(shell=False)
        self.main.plainTextEdit_log.setReadOnly(True)
        self.main.gridLayout_log.addWidget(self.main.plainTextEdit_log, 0, 0, 1, 1)


    #----------------------------------------------------------------------
    def enable_debugger(self):

        log = logging.getLogger()
        log.setLevel(logging.DEBUG)
        ch = logging.StreamHandler(self.main.plainTextEdit_log)
        formatter = logging.Formatter("[%(levelname)s] %(message)s")
        # formatter = logging.Formatter("[{levelname}] {message}")
        ch.setFormatter(formatter)
        log.addHandler(ch)


    #----------------------------------------------------------------------
    def write_log(self, data, prefix="[INFO]"):

        lines = ""
        if type(data) == type({}):
            line = key + ": " + kwargs[key]
            lines += line
        else:
            lines = data

        self.main.plainTextEdit_log.write(lines, prefix=prefix)



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os

import random
from PySide import QtCore, QtGui
import logging

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from configparser import RawConfigParser
    from io import StringIO
else:
    #Python2
    from ConfigParser import RawConfigParser
    from cStringIO import StringIO


from ..py_bloques import constructor as Blocks

from ...ide.methods.decorators import Decorator
from ...ide.methods.parser import remove_comments


########################################################################
class Code2Blocks(object):
    """"""


    #----------------------------------------------------------------------
    @Decorator.requiere_text_mode()
    def code_to_blocks(self, event=None):
        """"""

        code = self.parse_code()

        self.ide_new_file()
        start_pos = QtCore.QPoint(50, 150)

        work_area = self.ide.get_current_editor().graphical_area

        if code[1] != "{":
            last_block, pos = work_area.new_bloq("function", ['function', 'Pinguino', ['label', 'global'], ['syntax', ' variables']], QtCore.QPoint(50, 150), "global")
            last_block.metadata.pos_ = pos
            start_pos += QtCore.QPoint(500, 0)

        for line in range(len(code)):
            block_type = self.get_block_type(code, line)

            if code[line] in ["{", "}"]:
                continue

            constructor = self.get_contructor(block_type, code[line])
            block, pos = work_area.new_bloq(constructor[0], constructor, start_pos, "block_{}_{}".format(self.ide.get_current_filename(), line))
            block.metadata.pos_ = pos

            if block.metadata.type_ != "tipo4":
                new_pos = last_block.pos() + last_block.metadata.to_type[block.metadata.type_][0]

                if code[line-1] == "{":
                    nested = True
                else:
                    nested = None
                work_area.accept_move(True, block, last_block, nested)
                block.move(new_pos)

            else:
                start_pos += QtCore.QPoint(500, 0)

            work_area.expand_all()

            last_block = block


    #----------------------------------------------------------------------
    def parse_code(self):
        """"""
        code = self.ide.get_current_code()
        # code = self.ide.remove_comments(code)
        code = remove_comments(code)

        self.index_count = 0

        l = list(range(1, 10))
        l.reverse()
        code = code.replace("{", "\n{\n")
        code = code.replace("}", "\n}\n")
        code = code.replace(";", "\n")
        for i in l: code = code.replace(" "*i, " ")
        code = code.replace(", ", ",")

        code = code.split("\n")
        code = filter(lambda x:not x.isspace(), code)
        code = list(filter(None, code))

        for index in range(len(code)):
            if code[index].endswith(";"): code[index] = code[index][:-1]
            if code[index].startswith(" "): code[index] = code[index][1:]


        #fix for
        finish_for = False
        while not finish_for:
            for line in range(len(code)):
                try:
                    if code[line].startswith("for"):
                        code[line] = ";".join(code[line:line+3])
                        code.pop(line+1)
                        code.pop(line+1)

                    else:
                        finish_for = True

                except IndexError:
                    break




        return code


    #----------------------------------------------------------------------
    def get_block_type(self, code, line):
        """"""
        try:
            # Function
            if code[line+1] == "{" and code[line].split()[0] in ["char*", "char", "int", "float", "u8", "u16", "u32", "void"]:
                return Blocks.Function

            # Nested
            elif code[line+1] == "{" :
                return Blocks.Nested

            # Linear
            else:
                return Blocks.Linear

        except IndexError:
            return Blocks.Linear




    #----------------------------------------------------------------------
    def get_contructor(self, block, linecode):
        """"""
        blk = block("None")

        fill_args = []
        if linecode.count("("):
            name = linecode[:linecode.find("(")]
            args = linecode[linecode.find("(")+1:linecode.find(")")].split(",")


            syntax = ","

            if len(args) == 1:
                args = args[0].split(";")
                syntax = ";"

            # logging.debug(args)

            blk.addLabel(name)
            blk.addSyntax("(")

            for index in range(len(args)):
                if args[index]:

                    if len(args)>index>0:
                        blk.addSyntax(syntax)

                    if args[index] in ["False", "True"]:
                        blk.addSpaceBool()
                    else:
                        blk.addSpace()
                    fill_args.append(self.get_fancy_widget(args[index]))


            blk.addSyntax(")")
        else:
            blk.addLabel(linecode)

        if fill_args:
            blk.fillWith(fill_args)

        return blk.getBlock()


    #----------------------------------------------------------------------
    def get_fancy_widget(self, arg):
        """"""
        arg = self.fix_arg(arg)

        if arg in ["True", "False"]:
            return ("choice-B", arg)

        elif arg in ["INPUT", "OUTPUT"]:
            return ("choice-io", arg)

        elif arg in ["HIGH", "LOW"]:
            return ("choice-hl", arg)

        elif arg in ["GREENLED", "YELLOWLED", "USERBUTTON", "RUNLED", "USERLED"]:
            return ("choice-user", arg)

        elif arg in map(lambda x:"D{}".format(x), range(14)):
            return ("choice-D", arg)

        elif arg in map(lambda x:"A{}".format(x), range(6)):
            return ("choice-A", arg)

        elif arg.isdigit():
            return ("spin-int", arg)

        elif arg.count(".") == 1:
            try:
                type(eval(arg)) == float
                return ("spin-float", arg)
            except:
                return ("edit-value", arg)

        else:
            return ("edit-value", arg)



    #----------------------------------------------------------------------
    def fix_arg(self, arg):
        """"""
        data = {
            "int": ["int ", "u8 ", "u16 ", "u32 ", "u64 "],
            "float": ["float "],
            "string": ["char *"],
        }

        while arg.startswith(" "): arg = arg[1:]

        if filter(None, map(arg.startswith, data["int"])):
            return "{}".format(random.randint(0, 255))

        elif filter(None, map(arg.startswith, data["float"])):
            return "{:.2f}".format(random.random() + random.randint(0, 10))

        elif filter(None, map(arg.startswith, data["string"])):
            return "\"PinguiString\""




        return arg





#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import codecs
from datetime import datetime

from PySide import QtGui

from ..methods.decorators import Decorator

########################################################################
class Stdout(object):
    """"""

    #----------------------------------------------------------------------
    @Decorator.alert_tab("Stdout")
    def update_stdout(self):
        """"""
        file_ = os.path.join(os.getenv("PINGUINO_USER_PATH"), "source", "stdout")

        self.main.plainTextEdit_stdout.moveCursor(QtGui.QTextCursor.End)

        intro = "\n" * 2 + str(datetime.now()) + "\n" + "#" * 70 + "\n"
        self.main.plainTextEdit_stdout.insertPlainText(intro)

        if os.path.exists(file_):
            #RB20170215 : fixed a UnicodeDecodeError by adding errors='ignore'
            #Note : codecs.open() returns a file object that will encode unicode values to UTF-8
            #stdout = codecs.open(file_, "r", encoding="utf-8")
            stdout = codecs.open(file_, "r", encoding='utf-8', errors='ignore')
            content = stdout.readlines()
            stdout.close()
            self.main.plainTextEdit_stdout.insertPlainText("".join(content))
            self.main.plainTextEdit_stdout.moveCursor(QtGui.QTextCursor.EndOfLine)

        self.main.plainTextEdit_stdout.moveCursor(QtGui.QTextCursor.End)
        self.main.plainTextEdit_stdout.insertPlainText("\n"*4)







#! /usr/bin/python
#-*- coding: utf-8 -*-

"""
Semantic Versioning 2.0.0 - http://semver.org/

Given a version number MAJOR.MINOR.PATCH, increment the:

    MAJOR version when you make incompatible API changes,
    MINOR version when you add functionality in a backwards-compatible manner, and
    PATCH version when you make backwards-compatible bug fixes.

Additional labels for pre-release and build metadata are available as extensions to the MAJOR.MINOR.PATCH format.
"""

NAME = "Pinguino IDE"

MAJOR = "12"
MINOR = "0"
PATCH = "40"
PRE_RELEASE = "beta.2"

if PRE_RELEASE:
    VERSION = "{MAJOR}.{MINOR}.{PATCH}-{PRE_RELEASE}".format(**locals())
else:
    VERSION = "{MAJOR}.{MINOR}.{PATCH}".format(**locals())


"""-------------------------------------------------------------------------
    Pinguino IDE

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-------------------------------------------------------------------------"""

import os, sys

if sys.version_info >= (3, ):
    #Python3
    os.environ["PINGUINO_PYTHON"] = "3"
else:
    #Python2
    os.environ["PINGUINO_PYTHON"] = "2"

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    import imp
    imp.reload(sys)
else:
    reload(sys)
    sys.setdefaultencoding("utf-8")

os.environ["PINGUINO_NAME"] = NAME
os.environ["PINGUINO_VERSION"] = VERSION
os.environ["PINGUINO_FULLNAME"] = "{PINGUINO_NAME} {PINGUINO_VERSION}".format(**os.environ)
os.environ["PINGUINO_LIB"] = os.path.abspath(os.path.dirname(__file__))




#! /usr/bin/python2
#-*- coding: iso-8859-15 -*-

"""-------------------------------------------------------------------------
    Pinguino Bootloader Auto Detection

    Author:          Regis Blanchot <rblanchot@gmail.com>
    Last release:    2012-12-15

    This library is free software you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-------------------------------------------------------------------------"""

#import sys
#import os
#import usb            # checked in check.py

from uploader import baseUploader

class autodetect(baseUploader):
    """ detect bootloader version """

    # General Data Packet Structure (usbBuf)
    # --------------------------------------------------------------------------
    #    __________________
    #    |    COMMAND     |   0       [CMD]
    #    |      LEN       |   1       [LEN]
    #    |     ADDRL      |   2       [        ]  [addrl]
    #    |     ADDRH      |   3       [ADR.pAdr]: [addrh]
    #    |     ADDRU      |   4       [        ]  [addru]
    #    |                |   5       [DATA]
    #    |                |
    #    .      DATA      .
    #    .                .
    #    |                |   62
    #    |________________|   63
    #
    # --------------------------------------------------------------------------
    BOOT_CMD                        =    0
    BOOT_CMD_LEN                    =    1
    BOOT_ADDR_LO                    =    2
    BOOT_ADDR_HI                    =    3
    BOOT_ADDR_UP                    =    4
    BOOT_DATA_START                 =    5

    BOOT_DEV1                       =    5
    BOOT_DEV2                       =    6

    BOOT_VER_MINOR                  =    2
    BOOT_VER_MAJOR                  =    3

    BOOT_SIZE                       =    1

    # Bootloader commands
    # --------------------------------------------------------------------------
    READ_VERSION_CMD                =    0x00
    READ_FLASH_CMD                  =    0x01
    WRITE_FLASH_CMD                 =    0x02
    ERASE_FLASH_CMD                 =    0x03
    #READ_EEDATA_CMD                =    0x04
    #WRITE_EEDATA_CMD               =    0x05
    #READ_CONFIG_CMD                =    0x06
    #WRITE_CONFIG_CMD               =    0x07
    RESET_CMD                       =    0xFF

    # Data Block's size to write
    # --------------------------------------------------------------------------
    DATABLOCKSIZE                   =    32

    # USB Packet size
    # --------------------------------------------------------------------------
    MAXPACKETSIZE                   =    64

    # bulk endpoints
    # --------------------------------------------------------------------------
    IN_EP                           =    0x81    # endpoint for Bulk reads
    OUT_EP                          =    0x01    # endpoint for Bulk writes

    # configuration
    # --------------------------------------------------------------------------
    ACTIVE_CONFIG                   =    0x01
    INTERFACE_ID                    =    0x00
    TIMEOUT                         =    1200

# ------------------------------------------------------------------------------
    def initDevice(self):
# ------------------------------------------------------------------------------
#   TODO: to move in uploader.py ?
# ------------------------------------------------------------------------------
        """ init pinguino device """
        #conf = self.device.configurations[0]
        #iface = conf.interfaces[0][0]
        handle = self.device.open()
        if handle:
            ##handle.detachKernelDriver(iface.interfaceNumber)
            #handle.setConfiguration(conf)
            #handle.claimInterface(iface)
            ##handle.setAltInterface(iface)
            handle.setConfiguration(self.ACTIVE_CONFIG)
            handle.claimInterface(self.INTERFACE_ID)
            return handle
        return self.ERR_USB_INIT1
# ------------------------------------------------------------------------------
    def sendCMD(self, usbBuf):
# ------------------------------------------------------------------------------
        """ send command to the bootloader """
        sent_bytes = self.handle.bulkWrite(self.OUT_EP, usbBuf, self.TIMEOUT)
        #print sent_bytes
        if sent_bytes == len(usbBuf):
            #print "Block issued without problem."
            # whatever is returned, USB packet size is always 64 bytes long in high speed mode
            return self.handle.bulkRead(self.IN_EP, self.MAXPACKETSIZE, self.TIMEOUT)
        else:
            return self.ERR_USB_WRITE
# ------------------------------------------------------------------------------
    def detect(self):
# ------------------------------------------------------------------------------

        # search for a Pinguino board
        # ----------------------------------------------------------------------

        self.device = self.getDevice()
        if self.device == self.ERR_DEVICE_NOT_FOUND:
            self.add_report("Pinguino not found\n")
            self.add_report("Is your device connected and/or in bootloader mode ?\n")
            return
        else:
            self.add_report("Pinguino found ...\n")

        self.handle = self.initDevice()
        if self.handle == self.ERR_USB_INIT1:
            # wrong active config
            # bootloader version is 1.x or 2.x
            return (2,12)

        # find out bootloader version
        # ----------------------------------------------------------------------

        usbBuf = [0] * self.MAXPACKETSIZE
        # command code
        usbBuf[self.BOOT_CMD] = self.READ_VERSION_CMD
        # write data packet and get response
        usbBuf = self.sendCMD(usbBuf)
        if usbBuf != self.ERR_USB_WRITE:
            # major.minor
            version = ( usbBuf[self.BOOT_VER_MAJOR], \
                        usbBuf[self.BOOT_VER_MINOR] )
        self.closeDevice()
        return version
# ------------------------------------------------------------------------------



#!/usr/bin/env python
#-*- coding: utf-8 -*-

#import os

from PySide import QtCore


## Python3 compatibility
#if os.getenv("PINGUINO_PYTHON") is "3":
    ##Python3
    #from configparser import RawConfigParser
#else:
    ##Python2
    #from ConfigParser import RawConfigParser


from ..methods.dialogs import Dialogs


########################################################################
class Paths(object):

    def __init__(self):
        """"""

        self.load_paths()

        self.connect(self.main.lineEdit_path_sdcc_bin, QtCore.SIGNAL("editingFinished()"), self.save_paths)
        self.connect(self.main.lineEdit_path_gcc_bin, QtCore.SIGNAL("editingFinished()"), self.save_paths)
        self.connect(self.main.lineEdit_path_xc8_bin, QtCore.SIGNAL("editingFinished()"), self.save_paths)
        self.connect(self.main.lineEdit_path_8_libs, QtCore.SIGNAL("editingFinished()"), self.save_paths)
        self.connect(self.main.lineEdit_path_32_libs, QtCore.SIGNAL("editingFinished()"), self.save_paths)

        self.connect(self.main.pushButton_dir_sdcc, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_sdcc_bin))
        self.connect(self.main.pushButton_dir_gcc, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_gcc_bin))
        self.connect(self.main.pushButton_dir_xc8, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_xc8_bin))
        self.connect(self.main.pushButton_dir_8bit, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_8_libs))
        self.connect(self.main.pushButton_dir_32bit, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_32_libs))
        self.connect(self.main.pushButton_dir_libs, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_custom_libs))
        self.connect(self.main.pushButton_dir_mplab, QtCore.SIGNAL("clicked()"), lambda :self.open_path_for(self.main.lineEdit_path_mplab))

        self.connect(self.main.checkBox_paths_default, QtCore.SIGNAL("toggled(bool)"), self.set_default_values)


    #----------------------------------------------------------------------
    def open_path_for(self, lineedit):
        """"""
        dir_path = Dialogs.set_open_dir(self)
        if dir_path:
            lineedit.setText(dir_path)
            self.save_paths()


    #----------------------------------------------------------------------
    def save_paths(self):
        """"""
        sdcc_bin = self.main.lineEdit_path_sdcc_bin.text()
        gcc_bin = self.main.lineEdit_path_gcc_bin.text()
        xc8_bin = self.main.lineEdit_path_xc8_bin.text()
        p8libs = self.main.lineEdit_path_8_libs.text()
        p32libs = self.main.lineEdit_path_32_libs.text()

        #self.configIDE.set("Paths", "sdcc_bin", sdcc_bin)
        #self.configIDE.set("Paths", "gcc_bin", gcc_bin)
        #self.configIDE.set("Paths", "xc8_bin", xc8_bin)
        #self.configIDE.set("Paths", "pinguino_8_libs", p8libs)
        #self.configIDE.set("Paths", "pinguino_32_libs", p32libs)

        self.configIDE.set("PathsCustom", "sdcc_bin", sdcc_bin)
        self.configIDE.set("PathsCustom", "gcc_bin", gcc_bin)
        self.configIDE.set("PathsCustom", "xc8_bin", xc8_bin)
        self.configIDE.set("PathsCustom", "pinguino_8_libs", p8libs)
        self.configIDE.set("PathsCustom", "pinguino_32_libs", p32libs)

        self.configIDE.save_config()



    #----------------------------------------------------------------------
    def load_paths(self, section=None):
        """"""
        self.configIDE.load_config()
        if section is None:
            section = self.configIDE.config("Features", "pathstouse", "Paths")

        self.main.checkBox_paths_default.setChecked(section=="Paths")

        def short(section, option):
            if section == "Paths":
                return self.configIDE.get(section, option)
            elif section == "PathsCustom":
                return self.configIDE.config(section, option, self.configIDE.get("Paths", option))

        sdcc_bin = short(section, "sdcc_bin")
        gcc_bin = short(section, "gcc_bin")
        xc8_bin = short(section, "xc8_bin")
        p8libs = short(section, "pinguino_8_libs")
        p32libs = short(section, "pinguino_32_libs")

        self.main.lineEdit_path_sdcc_bin.setText(sdcc_bin)
        self.main.lineEdit_path_gcc_bin.setText(gcc_bin)
        self.main.lineEdit_path_xc8_bin.setText(xc8_bin)
        self.main.lineEdit_path_8_libs.setText(p8libs)
        self.main.lineEdit_path_32_libs.setText(p32libs)



    #----------------------------------------------------------------------
    def set_default_values(self, default):
        """"""

        self.configIDE.load_config()

        if default:
            self.load_paths(section="Paths")
            self.configIDE.set("Features", "pathstouse", "Paths")

        else:
            self.load_paths(section="PathsCustom")
            self.configIDE.set("Features", "pathstouse", "PathsCustom")


        self.main.groupBox_compilers.setEnabled(not default)
        self.main.groupBox_libraries.setEnabled(not default)
        self.main.groupBox_icsp.setEnabled(not default)

        self.configIDE.save_config()





#!/usr/bin/env python
#-*- coding: utf-8 -*-

from . import constructor as Block
from .constructor import Kit

Kit = Kit()

########################################################################
class LCD(object):

    LCD_noAutoScroll = lambda n:Kit.addFunct("LCD", "lcd.noAutoscroll", 0)
    LCD_autoscroll = lambda n:Kit.addFunct("LCD", "lcd.autoscroll", 0)
    LCD_rightToLeft = lambda n:Kit.addFunct("LCD", "lcd.rightToLeft", 0)
    LCD_leftToRight = lambda n:Kit.addFunct("LCD", "lcd.leftToRight", 0)
    LCD_scrollDisplayRight = lambda n:Kit.addFunct("LCD", "lcd.scrollDisplayRight", 0)
    LCD_scrollDisplayLeft = lambda n:Kit.addFunct("LCD", "lcd.scrollDisplayLeft", 0)

    LCD_blink = lambda n:Kit.addFunct("LCD", "lcd.blink", 0)
    LCD_noBlink = lambda n:Kit.addFunct("LCD", "lcd.noBlink", 0)
    LCD_cursor = lambda n:Kit.addFunct("LCD", "lcd.cursor", 0)
    LCD_noCursor = lambda n:Kit.addFunct("LCD", "lcd.noCursor", 0)
    LCD_display = lambda n:Kit.addFunct("LCD", "lcd.display", 0)
    LCD_noDisplay = lambda n:Kit.addFunct("LCD", "lcd.noDisplay", 0)
    LCD_clear = lambda n:Kit.addFunct("LCD", "lcd.clear", 0)
    LCD_home = lambda n:Kit.addFunct("LCD", "lcd.home", 0)

    LCD_printFloat = lambda n:Kit.addFunct("LCD", "lcd.printFloat", 1, fill=["spin-float"])
    LCD_print = lambda n:Kit.addFunct("LCD", "lcd.print", 1, fill=["edit"])
    LCD_printf = lambda n:Kit.addFunct("LCD", "lcd.printf", 2, fill=["edit", "more-args"])

    LCD_command = lambda n:Kit.addFunct("LCD", "lcd.command", 1)

    LCD_write = lambda n:Kit.addFunct("LCD", "lcd.write", 1, fill=["edit_c"])
    LCD_write = lambda n:Kit.addFunct("LCD", "lcd.write", 1, fill=["edit_c"])


    #----------------------------------------------------------------------
    def LCD_pins_8bit(self):

        pm = Block.Linear("LCD")
        pm.addLabel("lcd.pins")
        pm.addDecorator(" (8bits)")
        pm.addSyntax("(")

        pm.addArg("RS")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("E")
        pm.addSyntax(",")
        pm.addDecorator(" ")

        pm.addDecorator("D0-D7=")

        for i in range(7):
            pm.addSpace()
            pm.addSyntax(",")
        pm.addSpace()

        pm.fillWith(["spin-int"]*10)

        pm.addSyntax(")")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def LCD_pins_4bit(self):

        pm = Block.Linear("LCD")
        pm.addLabel("lcd.pins")
        pm.addDecorator(" (4bits)")
        pm.addSyntax("(")

        pm.addArg("RS")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("E")
        pm.addSyntax(",")
        pm.addDecorator(" ")

        pm.addDecorator("D4-D7=")

        for i in range(4):
            pm.addSpace()
            pm.addSyntax(",")

        for i in range(3):
            pm.addSyntax("0, ")
        pm.addSyntax("0")

        pm.fillWith(["spin-int"]*6)

        pm.addSyntax(")")
        return pm.getBlock()


    #----------------------------------------------------------------------
    def LCD_begin(self):

        pm = Block.Linear("LCD")
        pm.addLabel("lcd.begin")
        pm.addSyntax("(")
        pm.addArg("COL")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("ROW")
        pm.addSyntax(")")
        pm.addContextHelp("set up the LCD's number of columns and rows")
        pm.fillWith(["spin-int", "spin-int"])
        return pm.getBlock()


    #----------------------------------------------------------------------
    def LCD_setCursor(self):

        pm = Block.Linear("LCD")
        pm.addLabel("lcd.setCursor")
        pm.addSyntax("(")
        pm.addArg("COL")
        pm.addSyntax(",")
        pm.addDecorator(" ")
        pm.addArg("ROW")
        pm.addSyntax(")")
        pm.fillWith(["spin-int", "spin-int"])
        return pm.getBlock()


    #----------------------------------------------------------------------
    def LCD_printNumber(self):

        pm = Block.Linear("LCD")
        pm.addLabel("lcd.printNumber")
        pm.addSyntax("(")
        pm.addSpace()
        pm.addSyntax(",")
        pm.addArg(" base")
        pm.addSyntax(")")
        pm.fillWith(["spin-int", "number-base"])
        return pm.getBlock()



#! /usr/bin/python
#-*- coding: utf-8 -*-

try:
    from version import VERSION
except:
    from .version import VERSION


"""-------------------------------------------------------------------------
    Pinguino IDE

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-------------------------------------------------------------------------"""

import sys
import os

class BColors:
    Black = "\033[0;30m"
    Red = "\033[0;31m"
    Green = "\033[0;32m"
    Yellow = "\033[0;33m"
    Blue = "\033[0;34m"
    Purple = "\033[0;35m"
    Cyan = "\033[0;36m"
    LightGray = "\033[0;37m"
    ENDC = "\033[0m"

import argparse

try:
    from qtgui.pinguino_core.boards import boardlist
    from qtgui.pinguino_core.pinguino import Pinguino
    from qtgui.pinguino_core.pinguino_config import PinguinoConfig
    from qtgui.pinguino_core.config import Config
except:
    from .qtgui.pinguino_core.boards import boardlist
    from .qtgui.pinguino_core.pinguino import Pinguino
    from .qtgui.pinguino_core.pinguino_config import PinguinoConfig
    from .qtgui.pinguino_core.config import Config

pinguino = Pinguino()
PinguinoConfig.set_environ_vars()
PinguinoConfig.check_user_files()
config = Config()
PinguinoConfig.update_pinguino_paths(config, pinguino)
PinguinoConfig.update_pinguino_extra_options(config, pinguino)
PinguinoConfig.update_user_libs(pinguino)

def printb(text, color):
    print(color + text + BColors.ENDC)

#----------------------------------------------------------------------
def build_argparse():

    parser = argparse.ArgumentParser(description="*** {PINGUINO_NAME}:Command line ***".format(**os.environ))

    #command line args
    parser.add_argument("-v", "--version", dest="version", action="store_true", default=False, help="show {PINGUINO_NAME} version and exit".format(**os.environ))
    parser.add_argument("-a", "--author", dest="author", action="store_true", default=False, help="show authors of the {PINGUINO_NAME} and exit".format(**os.environ))
    parser.add_argument("-f", "--filename", dest="filename", nargs=1, default=False, help="filename to process")
    #parser.add_argument("-8c", "--8bit_compiler", dest="compiler", nargs=1, default=None, help="8-bit compiler: SDCC or XC8")
    parser.add_argument("--sdcc", dest="compiler_sdcc", action="store_true", default=None, help="Set SDCC as 8-bit compiler")
    parser.add_argument("--xc8", dest="compiler_xc8", action="store_true", default=None, help="Set XC8 as 8-bit compiler")
    #parser.add_argument("-l", "--boot", dest="bootloader", nargs=1, default=False, help="set bootloader option")
    parser.add_argument("--icsp", dest="icsp", action="store_true", default=False, help="use ICSP (default use bootloader)")
    parser.add_argument("-x", "--upload", dest="upload", action="store_true", default=False, help="upload code")
    parser.add_argument("-g", "--hex", dest="hex_file", action="store_true", default=False, help="print hex_file")

    for board in boardlist:
        parser.add_argument(board.shortarg, board.longarg, dest="board", const=board, action="store_const", default=False,
                            help="compile code for " + board.board + " board")

    parsed_c, others_c = parser.parse_known_args()

    if others_c:
        parser.print_help()
        sys.exit()
    else: return parsed_c

parser = build_argparse()

if parser.version:
    printb("\t" + VERSION, BColors.Green)
    sys.exit()

if parser.author:
    printb("\tJean-Pierre Mandon", BColors.Green)
    printb("\tRegis Blanchot", BColors.Green)
    printb("\tYeison Cardona", BColors.Green)
    sys.exit()

if parser.board:
    pinguino.set_board(parser.board)
    printb("using {} board".format(parser.board.name), BColors.Green)

    #if parser.bootloader:
        #bootloader = pinguino.dict_boot.get(parser.bootloader[0].lower(), parser.board.bldr)
        #pinguino.set_bootloader(*bootloader)
        #printb("using {} bootloader".format(pinguino.get_board().bldr), BColors.Green)

    if parser.icsp:
        #do nothing
        pinguino.set_icsp()  #use ICSP instead of bootloader
        printb("ICSP mode used", BColors.Green)
    else:
        printb("Bootloader {} used".format(pinguino.get_board().bldr), BColors.Green)


    if parser.board.arch == 8:
        if parser.compiler_sdcc:
            pinguino.set_8bit_compiler("sdcc")
        elif parser.compiler_xc8:
            pinguino.set_8bit_compiler("xc8")
        else:
            pinguino.set_8bit_compiler("xc8")


    if not parser.filename:
        printb("ERROR: missing filename", BColors.Red)
        sys.exit(1)

    else:
        filename = parser.filename

        if not (len(filename) == 1 and filename[0].endswith(".hex")):

            if len(filename) == 1:
                fname, extension = os.path.splitext(filename[0])
                if extension != ".pde":
                    printb("ERROR: bad file extension, it should be .pde", BColors.Red)
                    sys.exit()
                del fname, extension

            pinguino.compile_file(filename)

            if not pinguino.compiled():
                printb("\nERROR: not compiled\n", BColors.Red)

                ##errors_proprocess = pinguino.get_errors_preprocess()
                ##if errors_proprocess:
                    ##for error in errors_proprocess["preprocess"]: printb(error, BColors.Red)

                #errors_c = pinguino.get_errors_compiling_c()
                #if errors_c:
                    #printb(errors_c["complete_message"], BColors.Red)

                #errors_asm = pinguino.get_errors_compiling_asm()
                #if errors_asm:
                    #for error in errors_asm["error_symbols"]:
                        #printb(error, BColors.Red)

                #errors_link = pinguino.get_errors_linking()
                #if errors_link:
                    #for error in errors_link["linking"]:
                        #printb(error, BColors.Red)

                errors_c = pinguino.get_errors_compiling_c()
                if errors_c:
                    printb("ERROR: {complete_message}".format(**errors_c), BColors.Red)

                errors_asm = pinguino.get_errors_compiling_asm()
                if errors_asm:
                    printb("ERROR: {}".format("\n".join(errors_asm)), BColors.Red)

                errors_linking = pinguino.get_errors_linking()
                if errors_linking:
                    if "linking" in  errors_linking:
                        for error in errors_linking["linking"]:
                            printb("ERROR: {}".format(error), BColors.Red)

                sys.exit()

            else:
                result = pinguino.get_result()
                printb("compilation time: {time}".format(**result), BColors.Yellow)
                printb("compiled to: {hex_file}".format(**result), BColors.Yellow)

                if parser.hex_file:
                    hex_file = open(result["hex_file"], "r")
                    content_hex = hex_file.readlines()
                    hex_file.close()
                    printb("\n" + "*" * 70, BColors.Cyan)
                    printb(result["hex_file"], BColors.Cyan)
                    printb("*" * 70, BColors.Cyan)
                    for line in content_hex: printb(line, BColors.Cyan),
                    printb("*" * 70 + "\n", BColors.Cyan)

                print "OK"
        else:
            pinguino.__hex_file__ = filename[0]

        if parser.upload:
            try:
                uploaded, result = pinguino.upload()
                if result:
                    printb(result, BColors.Green)
            except Exception as err:
                printb(err, BColors.Red)
                #if pinguino.get_board().arch == 8:
                    #printb("ERROR: bootloader option could be incorrect.", BColors.Red)
                    #printb("Bootloader options: ", BColors.Green),
                    #printb(", ".join(pinguino.dict_boot.keys()), BColors.Green)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/yeison/Documentos/Desarrollo/Pinguino/GitHub/pinguino-ide/pinguino/qtgui/gide/bloques/inside/inside2_bool.ui'
#
# Created: Wed Mar 16 13:19:41 2016
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(94, 34)
        Form.setWindowTitle("")
        self.gridLayout = QtGui.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.frame = QtGui.QFrame(Form)
        self.frame.setMinimumSize(QtCore.QSize(9, 34))
        self.frame.setMaximumSize(QtCore.QSize(9, 34))
        self.frame.setObjectName("frame")
        self.gridLayout.addWidget(self.frame, 0, 0, 1, 1)
        self.frame_2 = QtGui.QFrame(Form)
        self.frame_2.setMinimumSize(QtCore.QSize(0, 34))
        self.frame_2.setMaximumSize(QtCore.QSize(16777215, 34))
        self.frame_2.setObjectName("frame_2")
        self.gridLayout.addWidget(self.frame_2, 0, 1, 1, 1)
        self.frame_3 = QtGui.QFrame(Form)
        self.frame_3.setMinimumSize(QtCore.QSize(9, 34))
        self.frame_3.setMaximumSize(QtCore.QSize(9, 34))
        self.frame_3.setObjectName("frame_3")
        self.gridLayout.addWidget(self.frame_3, 0, 2, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        pass




#! /usr/bin/python2
#-*- coding: utf-8 -*-

from PySide import QtGui, QtCore

########################################################################
class PrettyFeatures(object):


    #----------------------------------------------------------------------
    @classmethod
    #def LineEdit_default_text(self, parent, lineEdit, default_text):
    def LineEdit_default_text(cls, lineEdit, default_text):


        def set_focus_out_color():
            lineEdit.setStyleSheet("color: #aaa;")

        def set_focus_in_color():
            lineEdit.setStyleSheet("")

        def focus_in(event):
            if lineEdit.text() == default_text:
                lineEdit.setText("")
                set_focus_in_color()
            QtGui.QLineEdit.focusInEvent(lineEdit, QtGui.QFocusEvent(QtCore.QEvent.FocusIn))

        def focus_out(event):
            if lineEdit.text() == "":
                lineEdit.setText(default_text)
                set_focus_out_color()
            QtGui.QLineEdit.focusOutEvent(lineEdit, QtGui.QFocusEvent(QtCore.QEvent.FocusOut))

        lineEdit.focusInEvent = focus_in
        lineEdit.focusOutEvent = focus_out

        lineEdit.setText(default_text)
        set_focus_out_color()




#! /usr/bin/python2
#-*- coding: iso-8859-15 -*-

"""-------------------------------------------------------------------------
    Pinguino Universal Uploader

    (c) 2011 Regis Blanchot <rblanchot@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-------------------------------------------------------------------------"""

# This class is based on :
# - Docker software licensed (LGPL) by Pierre Gaufillet <pierre.gaufillet@magic.fr>
# - Vasco  software licensed (LGPL) by Jean-Pierre Mandon <jp.mandon@gmail.com>
# PyUSB Doc can be found here: http://wiki.erazor-zone.de/wiki:projects:python:pyusb:pydoc
# To get the Device Descriptors : lsusb -v -d 04d8:feaa

import sys
import os
#import usb			# checked in check.py

from .uploader import baseUploader

class uploaderVSC(baseUploader):
    """ upload .hex into pinguino device """

    # Vasco Bulk bootloader commands
    # --------------------------------------------------------------------------

    VSC_ERASE_FLASH_CMD				=	0x00
    VSC_WRITE_FLASH_CMD				=	0x01
    VSC_READ_FLASH_CMD				=	0x02
    VSC_VALID_APPLICATION_CMD		=	0x03
    VSC_RESET_DEVICE_CMD			=	0x04
    VSC_SECTION_DESCRIPTOR_CMD		=	0x05

    # boot command size in bytes
    VSC_BOOT_CMD_SIZE				=	4
    VSC_BLOCKSIZE					=	32

    # bulk endpoints
    VSC_IN_EP						=	0x82	# endpoint for Bulk reads
    VSC_OUT_EP						=	0x01	# endpoint for Bulk writes

    # configuration
    VSC_TIMEOUT						=	200

    # Table with supported USB devices
    # device_id:[PIC name, flash size(in bytes), eeprom size (in bytes)]
    # --------------------------------------------------------------------------

    devices_table = \
    {
        0x1260: ['18f2455'],
        0x1240: ['18f2550'],
        0x1220: ['18f4455'],
        0x1200: ['18f4550']
    }

# ------------------------------------------------------------------------------
    def usbWrite(self, handle, usbBuf):
# ------------------------------------------------------------------------------
        """	Write a data packet to currently-open USB device """
        sent_bytes = handle.bulkWrite(self.VSC_OUT_EP, usbBuf, self.VSC_TIMEOUT)
        if (sent_bytes):
            return self.ERR_NONE
        else:
            return self.ERR_USB_WRITE
# ------------------------------------------------------------------------------
    def eraseBlock(self, handle, address):
# ------------------------------------------------------------------------------
        """ erase 64 bytes of flash memory """
        # command
        cmd = self.VSC_ERASE_FLASH_CMD
        # block address
        address = "%06X" % address
        addr_lo = int(address[4:6],16)
        addr_hi = int(address[2:4],16)
        addr_up = int(address[0:2],16)
        # write data packet
        usbBuf = chr(cmd) + chr(addr_lo) + chr(addr_hi) + chr(addr_up)
        self.usbWrite(handle, usbBuf)
# ------------------------------------------------------------------------------
    #def issueBlock(self, address, block):
    def writeFlash(self, handle, address, datablock):
# ------------------------------------------------------------------------------
        """ write a block of code """
        # command
        cmd = self.VSC_WRITE_FLASH_CMD
        # block's address
        address = "%06X" % address
        readbyte1 = int(address[4:6], 16)
        readbyte2 = int(address[2:4], 16)
        readbyte3 = int(address[0:2], 16)
        # add data to the packet
        usbBuf = chr(cmd) + chr(readbyte1) + chr(readbyte2) + chr(readbyte3)
        for i in range(len(datablock)):
            usbBuf = usbBuf + chr(datablock[i])
        # write data packet on usb device
        self.usbWrite(handle, usbBuf)
# ------------------------------------------------------------------------------
    #def hexWrite(self):
    def writeHex(self, handle, filename, board):
# ------------------------------------------------------------------------------
        """ Parse the Hex File Format and send data to usb device """

        """
        [0]		Start code, one character, an ASCII colon ':'.
        [1:3]	Byte count, two hex digits, a number of bytes (hex digit pairs) in the data field. 16 (0x10) or 32 (0x20) bytes of data are the usual compromise values between line length and address overhead.
        [3:7]	Address, four hex digits, a 16-bit address of the beginning of the memory position for the data. Limited to 64 kilobytes, the limit is worked around by specifying higher bits via additional record types. This address is big endian.
        [7:9]	Record type, two hex digits, 00 to 05, defining the type of the data field.
        [9:*]	Data, a sequence of n bytes of the data themselves, represented by 2n hex digits.
        [*:*]	Checksum, two hex digits - the least significant byte of the two's complement of the sum of the values of all fields except fields 1 and 6 (Start code ":" byte and two hex digits of the Checksum). It is calculated by adding together the hex-encoded bytes (hex digit pairs), then leaving only the least significant byte of the result, and making a 2's complement (either by subtracting the byte from 0x100, or inverting it by XOR-ing with 0xFF and adding 0x01). If you are not working with 8-bit variables, you must suppress the overflow by AND-ing the result with 0xFF. The overflow may occur since both 0x100-0 and (0x00 XOR 0xFF)+1 equal 0x100. If the checksum is correctly calculated, adding all the bytes (the Byte count, both bytes in Address, the Record type, each Data byte and the Checksum) together will always result in a value wherein the least significant byte is zero (0x00).
                For example, on :0300300002337A1E
                03 + 00 + 30 + 00 + 02 + 33 + 7A = E2, 2's complement is 1E
        """

        data = []
        old_address = 0
        max_address = 0
        address_Hi	= 0
        codesize	= 0

        # read hex file
        # ----------------------------------------------------------------------

        fichier = open(filename,'r')
        lines = fichier.readlines()
        fichier.close()

        # 1st pass : calculate checksum and max address
        # ----------------------------------------------------------------------

        i = 0
        for line in lines:
            byte_count = int(line[1:3], 16)
            address_Lo = int(line[3:7], 16) # lower 16 bits (bits 0-15) of the data address
            record_type= int(line[7:9], 16)
            i+=1

            address = (address_Hi << 16) + address_Lo

            # checksum calculation
            end = 9 + byte_count * 2 # position of checksum at end of line
            checksum = int(line[end:end+2], 16)
            cs = 0
            i = 1
            while i < end:
                cs = cs + (0x100 - int(line[i:i+2], 16) ) & 0xff # not(i)
                i = i + 2
            if checksum != cs:
                return self.ERR_HEX_CHECKSUM

            # extended linear address record
            if record_type == self.Extended_Linear_Address_Record:
                address_Hi = int(line[9:13], 16) << 16 # upper 16 bits (bits 16-31) of the data address

            # code size
            if address >= board.memstart:
                codesize = codesize + byte_count

            # max address
            if (address > old_address) and (address < board.memend):
                max_address = address + byte_count
                old_address = address

        max_address = max_address + 64 - (max_address % 64)
        #self.txtWrite(output, "%d bytes to write\n" % codesize)

        # fill data sequence with 0xFF
        # ----------------------------------------------------------------------

        for i in range(board.memstart, max_address + 64):
            #print i
            data.append(0xFF)

        # 2nd pass : parse bytes from line into data
        # ----------------------------------------------------------------------

        address_Hi	= 0

        for line in lines:
            byte_count = int(line[1:3], 16)
            address_Lo = int(line[3:7], 16) # four hex digits
            record_type= int(line[7:9], 16)

            address = (address_Hi << 16) + address_Lo

            # data record
            if record_type == self.Data_Record:
                if (address >= board.memstart) and (address < board.memend):
                    for i in range(byte_count):
                        data[address - board.memstart + i] = int(line[9 + (2 * i) : 11 + (2 * i)], 16)

            # end of file record
            elif record_type == self.End_Of_File_Record:
                break

            # extended linear address record
            elif record_type == self.Extended_Linear_Address_Record:
                address_Hi = int(line[9:13], 16) # upper 16 bits (bits 16-31) of the data address

            # unsupported record type
            else:
                return self.ERR_HEX_RECORD

        # erase and write blocks
        # ----------------------------------------------------------------------

        usbBuf = []
        for addr in range(board.memstart, max_address + 64):
            if addr % 64 == 0:
                self.eraseBlock(handle, addr)
            if addr % self.VSC_BLOCKSIZE == 0:
                if usbBuf != []:
                    self.writeFlash(handle, addr - self.VSC_BLOCKSIZE, usbBuf)
                usbBuf = []
            if data[addr - board.memstart] != []:
                usbBuf.append(data[addr - board.memstart])

        return self.ERR_NONE
# ------------------------------------------------------------------------------
    #def writeHex(self):
    def uploadDevice(self, filename, board):
# ------------------------------------------------------------------------------

        # check file to upload
        # ----------------------------------------------------------------------

        if filename == '':
            self.add_report("No program to write")
            self.closeDevice(handle)
            return

        fichier = open(filename, 'r')
        if fichier == "":
            self.add_report("Unable to open %s" % filename)
            return
        fichier.close()

        # search for a Pinguino board
        # ----------------------------------------------------------------------

        device = self.getDevice(board)
        if device is self.ERR_DEVICE_NOT_FOUND:
            self.add_report("Pinguino not found")
            self.add_report("If your device is connected,")
            self.add_report("press the Reset button to switch to bootloader mode.")
            return
        else:
            self.add_report("Pinguino found")

        handle = self.initDevice(device)


        if handle is self.ERR_USB_INIT1:
            self.add_report("Upload not possible")
            self.add_report("Try to restart the bootloader mode")
            return

        # find out bootloader version
        # ----------------------------------------------------------------------

        #product = handle.getString(device.iProduct, 30)
        #manufacturer = handle.getString(device.iManufacturer, 30)
        #self.add_report("Bootloader v2.12")

        # start writing
        # ----------------------------------------------------------------------

        #self.add_report("Writing ...")
        #status = self.hexWrite()
        status = self.writeHex(handle, filename, board)
        if status == self.ERR_NONE:
            self.add_report(os.path.basename(filename) + " successfully uploaded")
        if status == self.ERR_HEX_RECORD:
            self.add_report("Record error")
        if status == self.ERR_HEX_CHECKSUM:
            self.add_report("Checksum error")

        self.closeDevice(handle)
# ------------------------------------------------------------------------------



#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os

from PySide import QtGui, QtCore

from ...frames.insert_block import Ui_InsertBlock

########################################################################
class InsertBlock(QtGui.QDialog):

    def __init__(self, KIT):
        super(InsertBlock, self).__init__()

        self.insert = Ui_InsertBlock()
        self.insert.setupUi(self)

        self.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+self.windowTitle())

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/logo/art/windowIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        self.graphical = KIT

        self.connect(self.insert.lineEdit, QtCore.SIGNAL("textChanged(QString)"), self.update_blocks)
        self.connect(self.insert.listWidget, QtCore.SIGNAL("itemActivated(QListWidgetItem*)"), self.insert_block)
        #self.connect(self.insert.listWidget, QtCore.SIGNAL("itemEntered(QListWidgetItem*)"), self.insert_block)

        self.insert.lineEdit.keyPressEvent = self.line_edit_key
        self.insert.listWidget.keyPressEvent = self.list_key

        self.insert.lineEdit.setText(QtGui.QApplication.translate("Dialogs", "search block..."))
        self.insert.lineEdit.selectAll()

        self.setStyleSheet("""
        font-family: inherit;
        font-weight: normal;

        """)

        self.center_on_screen()


    #----------------------------------------------------------------------
    def center_on_screen(self):

        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)

    #----------------------------------------------------------------------
    def line_edit_key(self, event):

        key = event.key()
        if key in (QtCore.Qt.Key_Down, ):
            self.insert.listWidget.setFocus()
            self.insert.listWidget.setCurrentRow(0)

        if key in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Enter - 1):
            if self.insert.listWidget.count() == 1:
                self.insert_block(self.insert.listWidget.item(0))

        QtGui.QLineEdit.keyPressEvent(self.insert.lineEdit, event)


    #----------------------------------------------------------------------
    def list_key(self, event):

        key = event.key()
        if key in (QtCore.Qt.Key_Down,
                   QtCore.Qt.Key_Up,
                   QtCore.Qt.Key_Enter,
                   QtCore.Qt.Key_Enter - 1,
                   ):
            QtGui.QListWidget.keyPressEvent(self.insert.listWidget, event)
            return
        QtGui.QLineEdit.keyPressEvent(self.insert.lineEdit, event)


    #----------------------------------------------------------------------
    def update_blocks(self, text):

        bloques = []
        self.items = {}
        self.all_sets = self.graphical.get_all_sets()
        for key in self.all_sets.keys():
            if self.all_sets[key][2][0] in ["label", "decorator"]:
                label = self.all_sets[key][2][1]
                if label.lower().startswith(text.lower()):
                    bloques.append([key, self.all_sets[key]])
                    self.items[label] = [key, self.all_sets[key]]
        self.insert.listWidget.clear()
        self.insert.listWidget.addItems(self.items.keys())


    #----------------------------------------------------------------------
    def insert_block(self, list_widget):

        self.hide()
        work_area = self.graphical.get_work_area()
        block = self.items[list_widget.text()]
        name = block[1][0]
        args = block[1][:]
        baseName = list_widget.text()
        args[0] = ""
        args[1] = ""
        pos = self.cursor().pos() - QtCore.QPoint(10, 10)
        if name: work_area.new_bloq(name, args, pos, baseName)
        self.close()




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import logging
import shutil
import platform
import sys

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from configparser import RawConfigParser
    from os import makedirs
else:
    #Python2
    from ConfigParser import RawConfigParser

    #----------------------------------------------------------------------
    def makedirs(name, mode=0o777, exist_ok=False):
        """makedirs(name [, mode=0o777][, exist_ok=False])

        Super-mkdir; create a leaf directory and all intermediate ones.  Works like
        mkdir, except that any intermediate path segment (not just the rightmost)
        will be created if it does not exist. If the target directory already
        exists, raise an OSError if exist_ok is False. Otherwise no exception is
        raised.  This is recursive.

        """
        head, tail = os.path.split(name)
        if not tail:
            head, tail = os.path.split(head)
        if head and tail and not os.path.exists(head):
            try:
                makedirs(head, mode, exist_ok)
            except FileExistsError:
                # Defeats race condition when another thread created the path
                pass
            cdir = os.curdir
            if isinstance(tail, bytes):
                cdir = bytes(os.curdir, 'ASCII')
            if tail == cdir:           # xxx/newdir/. exists if xxx/newdir exists
                return
        try:
            os.mkdir(name, mode)
        except OSError:
            # Cannot rely on checking for EEXIST, since the operating system
            # could give priority to other errors like EACCES or EROFS
            if not exist_ok or not os.path.isdir(name):
                raise



########################################################################
class PinguinoConfig(object):

    #----------------------------------------------------------------------
    @classmethod
    def set_environ_vars(cls):
        """Configure some environ variables about OS and architecture.
        """

        #if not os.path.exists(os.path.join(os.getenv("PINGUINO_LIB"), "paths.cfg")):
            #logging.error("Missing: "+os.path.join(os.getenv("PINGUINO_LIB"), "paths.cfg"))
            #sys.exit()

        #config_paths = RawConfigParser()
        #config_paths.readfp(open(os.path.join(os.getenv("PINGUINO_LIB"), "paths.cfg"), "r")


        #RB20141116 : get the “bitness” of the current OS
        bitness, linkage = platform.architecture()
        os.environ["PINGUINO_OS_ARCH"] = bitness

        if os.name == "posix": #GNU/Linux
            os.environ["PINGUINO_OS_NAME"] = "linux"

        #Mac could return posix :/
        elif os.name == "os2":  #Mac OS X
            os.environ["PINGUINO_OS_NAME"] = "macosx"

        elif os.name == "nt":  #Windows
            os.environ["PINGUINO_OS_NAME"] = "windows"

        config_paths = RawConfigParser()
        config_paths.readfp(open(os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "config", "pinguino.{PINGUINO_OS_NAME}.conf".format(**os.environ)), "r"))

        #load path from paths.conf
        os.environ["PINGUINO_USER_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("Paths", "user_path")))
        os.environ["PINGUINO_INSTALL_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("Paths", "install_path")))
        os.environ["PINGUINO_USERLIBS_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("Paths", "user_libs")))

        os.environ["PINGUINO_LIBS_PATH"] = os.path.expandvars(os.path.expanduser(os.path.dirname(config_paths.get("Paths", "pinguino_8_libs"))))

        os.environ["PINGUINO_8_LIBS_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("Paths", "pinguino_8_libs")))
        os.environ["PINGUINO_32_LIBS_PATH"] = os.path.expandvars(os.path.expanduser(config_paths.get("Paths", "pinguino_32_libs")))

        # os.environ["PINGUINO_USERLIBS_PATH"] = os.path.expandvars(os.path.join(os.getenv("PINGUINO_USER_PATH"), "libraries"))
        os.environ["PINGUINO_DEFAULT_FILES"] = os.path.join(os.getenv("PINGUINO_USER_PATH"), "local")


    #----------------------------------------------------------------------
    @classmethod
    def check_user_files(cls):
        """Check existence of depends directories and created if missed."""

        #check instalation path
        if not os.path.exists(os.getenv("PINGUINO_INSTALL_PATH")):
            logging.warning("Missing Pinguino libraries instalation. "
                            "Make sure the paths in %s are correct."%os.path.join(os.getenv("PINGUINO_LIB"), "paths.conf"))

        #create ~/.pinguino
        if not os.path.exists(os.getenv("PINGUINO_USER_PATH")):
            makedirs(os.getenv("PINGUINO_USER_PATH"), exist_ok=True)

        #create ~/.pinguino/user
        if not os.path.exists(os.getenv("PINGUINO_DEFAULT_FILES")):
            makedirs(os.getenv("PINGUINO_DEFAULT_FILES"), exist_ok=True)

        #Check files and directories
        cls.check_locations()


    #----------------------------------------------------------------------
    @classmethod
    def check_locations(cls):
        """If not exist some directory then copy them from installation path."""

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_LIBS_PATH"), "examples"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "examples"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_LIBS_PATH"), "graphical_examples"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "graphical_examples"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_LIBS_PATH"), "source"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "source"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "p32", "obj"),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "source", "obj"))

        cls.if_not_exist_then_copy(src=os.path.join(os.getenv("PINGUINO_LIB"), "qtgui", "config", "pinguino.%s.conf"%os.getenv("PINGUINO_OS_NAME")),
                                   dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "pinguino.conf"))


        if os.path.exists(os.getenv("PINGUINO_USERLIBS_PATH")):
            for lib in os.listdir(os.getenv("PINGUINO_USERLIBS_PATH")):
                src = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), lib, "examples")
                if os.path.exists(src):
                    cls.if_not_exist_then_copy(src=src,
                                               dst=os.path.join(os.getenv("PINGUINO_USER_PATH"), "examples_libs", lib))


    #----------------------------------------------------------------------
    @classmethod
    def if_not_exist_then_copy(cls, src, dst):
        """The method name say all.

        Parameters
        ----------
        src: str
            Source file or directory.

        dst: str
            Destiny file or directory.
        """
        if not os.path.exists(src):
            return


        #When src is a file
        if os.path.isfile(src):
            if not os.path.exists(src): logging.warning("Missing: " + src)
            else:
                if not os.path.exists(dst): shutil.copy(src, dst)
            return

        #Create parent directories
        def prepare_directory(file_path):
            dir_, filename = os.path.split(file_path)
            all_dirs = []
            while not os.path.exists(dir_):
                all_dirs.append(dir_)
                dir_, dir2 = os.path.split(dir_)
            all_dirs.reverse()
            for dir_ in all_dirs: makedirs(dir_, exist_ok=True)

        #When src is a directory, copy file by file
        all_files = []
        for path, dirs, files in os.walk(src):
            for f in files:
                file_path_src = os.path.join(path, f)
                file_path_dst = dst + os.path.join(path, f).replace(src, "")

                if not os.path.exists(file_path_dst):
                    logging.warning("Copying: " + file_path_dst)
                    prepare_directory(file_path_dst)
                    shutil.copy(file_path_src, file_path_dst)


    #----------------------------------------------------------------------
    @classmethod
    def update_pinguino_extra_options(self, config, pinguino_object):
        """Load on Pinguino Object the advance configuration for 32 bit.

        Parameters
        ----------
        config: RawConfigParser
            Config file.
        pinguino_object: Pinguino()
            Pinguino
        """

        pinguino_object.MIPS16 = str(config.config("Board", "mips16", True)).lower()
        pinguino_object.HEAPSIZE = str(config.config("Board", "heapsize", 512))
        pinguino_object.OPTIMIZATION = config.config("Board", "optimization", "-O3")


    #----------------------------------------------------------------------
    @classmethod
    def update_pinguino_paths(cls, config, pinguino_object, prefix=None, workspace=None):
        """Load paths of libraries and compilers.

        Parameters
        ----------
        config: RawConfigParser
            Config file.
        pinguino_object: Pinguino()
            Pinguino
        prefix: str, optional
            Path for search libraries and compilers, default is absolute path from root.
        workspace: str,optional
            Path for makefiles and temp files, default is ~/.pinguino/source

        """

        user_sdcc_bin = config.get_path("sdcc_bin", prefix)
        if user_sdcc_bin: pinguino_object.P8_SDCC_BIN = user_sdcc_bin
        #else: logging.warning("Missing path to 'sdcc_bin': '%s'"%user_sdcc_bin)

        user_sdcc_bin = config.get_path("xc8_bin", prefix)
        if user_sdcc_bin: pinguino_object.P8_XC8_BIN = user_sdcc_bin
        #else: logging.warning("Missing path to 'xc8_bin': '%s'"%user_sdcc_bin)

        user_gcc_bin = config.get_path("gcc_bin", prefix)
        if user_gcc_bin: pinguino_object.P32_GCC_BIN = user_gcc_bin
        #else: logging.warning("Missing path to 'gcc_bin': '%s'"%user_gcc_bin)

        if workspace:
            pinguino_object.SOURCE_DIR = workspace
        else:
            pinguino_source = os.path.join(os.getenv("PINGUINO_USER_PATH"), "source")
            #pinguino_source = os.path.join(os.path.dirname(os.getenv("PINGUINO_8_LIBS_PATH")), "source")
            if pinguino_source: pinguino_object.SOURCE_DIR = pinguino_source
            else: logging.warning("Missing path to 'source': '%s'"%pinguino_source)

        pinguino_8_libs = config.get_path("pinguino_8_libs", prefix)
        if pinguino_8_libs: pinguino_object.P8_DIR = pinguino_8_libs
        else: logging.warning("Missing path to '8_libs': '%s'"%pinguino_8_libs)

        pinguino_32_libs = config.get_path("pinguino_32_libs", prefix)
        if pinguino_32_libs: pinguino_object.P32_DIR = pinguino_32_libs
        else: logging.warning("Missing path to '32_libs': '%s'"%pinguino_32_libs)


    #----------------------------------------------------------------------
    @classmethod
    def update_user_libs(cls, pinguino_object, p8=[], p32=[], pdl=[]):
        """Reload paths for user installed libraries.

        Parameters
        ----------
        pinguino_object: Pinguino()
            Pinguino
        """

        pinguino_object.USER_P8_LIBS = cls.get_p8_libraries() + p8
        pinguino_object.USER_P32_LIBS = cls.get_p32_libraries() + p32
        pinguino_object.USER_PDL = cls.get_pdl_libraries() + pdl


    #----------------------------------------------------------------------
    @classmethod
    def get_p8_libraries(cls):
        """
        This is path where are .c files.

        """
        p8_libs = []
        if os.path.exists(os.getenv("PINGUINO_USERLIBS_PATH")):
            for lib in os.listdir(os.getenv("PINGUINO_USERLIBS_PATH")):
                p8 = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), lib, "p8")
                if os.path.exists(p8):
                    p8_libs.append(p8)

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os.getenv("PINGUINO_P8", False):
            if os_name == "windows":
                p8_libs.extend(os.getenv("PINGUINO_P8").split(";"))
            elif os_name == "linux":
                p8_libs.extend(os.getenv("PINGUINO_P8").split(":"))
            #FIXME_OSX
            # elif os_name == "macosx":
                # p32_libs.extend(os.getenv(PINGUINO_P32).aplit(":"))


        return p8_libs


    #----------------------------------------------------------------------
    @classmethod
    def get_p32_libraries(cls):
        """

        This is path where are .c files.

        """
        p32_libs = []
        if os.path.exists(os.getenv("PINGUINO_USERLIBS_PATH")):
            for lib in os.listdir(os.getenv("PINGUINO_USERLIBS_PATH")):
                p32 = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), lib, "p32")
                if os.path.exists(p32):
                    p32_libs.append(p32)

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os.getenv("PINGUINO_P32", False):
            if os_name == "windows":
                p32_libs.extend(os.getenv("PINGUINO_P32").split(";"))
            elif os_name == "linux":
                p32_libs.extend(os.getenv("PINGUINO_P32").split(":"))
            #FIXME_OSX
            # elif os_name == "macosx":
                # p32_libs.extend(os.getenv(PINGUINO_P32).aplit(":"))

        return p32_libs


    #----------------------------------------------------------------------
    @classmethod
    def get_pdl_libraries(cls):
        """

        This is individual paths for each .pdl or pdl32.

        """

        pdl_libs = []
        if os.path.exists(os.getenv("PINGUINO_USERLIBS_PATH")):
            for lib in os.listdir(os.getenv("PINGUINO_USERLIBS_PATH")):
                pdls = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), lib, "pdl")
                if os.path.exists(pdls):
                    for pdl in os.listdir(pdls):
                        if pdl.endswith(".pdl") or pdl.endswith(".pdl32"):
                            pdl_libs.append(os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), lib, "pdl", pdl))

        os_name = os.getenv("PINGUINO_OS_NAME")
        if os.getenv("PINGUINO_PDL", False):
            if os_name == "windows":
                pdl_libs.extend(os.getenv("PINGUINO_PDL").split(";"))
            elif os_name == "linux":
                pdl_libs.extend(os.getenv("PINGUINO_PDL").split(":"))
            #FIXME_OSX
            # elif os_name == "macosx":
                # pdl_libs.extend(os.getenv(PINGUINO_P32).aplit(":"))


        return pdl_libs




#!/usr/bin/env python
#-*- coding: utf-8 -*

import os
import sys
from setuptools import setup
from . import version

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    from configparser import RawConfigParser
else:
    #Python2
    from ConfigParser import RawConfigParser


from .qtgui.pinguino_core.pinguino_config import PinguinoConfig
PinguinoConfig.set_environ_vars()


########################################################################
class PinguinoLib:
    """"""

    #----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""

        config_file = os.path.join(sys.path[0], "PINGUINO")

        self.PINGUINO = RawConfigParser()
        self.PINGUINO.readfp(open(config_file, "r"))
        self.PINGUINO = dict(self.PINGUINO.items("PINGUINO"))

        self.BASE_DIR = sys.path[0]



    #----------------------------------------------------------------------
    def get_data_files(self):

        PINGUINOLIBS_DIR = os.path.join(os.getenv("PINGUINO_USERLIBS_PATH"), self.PINGUINO["name"])
        data_files = []
        mainlib = [(PINGUINOLIBS_DIR, "PINGUINO")]
        for dir_ in ["p8", "p32", "pdl", "examples"]:
            path = os.path.join(self.BASE_DIR, dir_)
            if os.path.exists(path):
                for file_ in os.listdir(path):
                    # mainlib.append(os.path.join(dir_, file_))
                    data_files.append((os.path.join(PINGUINOLIBS_DIR, dir_), [os.path.join(dir_, file_)]))

        return data_files


    #----------------------------------------------------------------------
    def pinguino_setup(self, **kwargs):
        """"""
        return setup(install_requires = ["pinguino",
                                         ],

                    # include_package_data = True,
                    data_files = self.get_data_files(),
                    zip_safe = False,


                    **kwargs
                    )


    #----------------------------------------------------------------------
    @classmethod
    def get_setup_template(cls):
        return """
#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
from pinguino.dev import PinguinoLib

library = PinguinoLib()
PINGUINO = library.PINGUINO

library.pinguino_setup(

    name = "pinguino-{name}".format(**PINGUINO),
    version = PINGUINO["version"],
    description = PINGUINO["description"],

    author = PINGUINO["author"],
    author_email = PINGUINO["author_email"],
    maintainer = "",
    maintainer_email = "",

    url = PINGUINO["url"],
    download_url = "",

    license = "",
    keywords = "microchip, electronic, prototyping, pinguino",

    classifiers=[#list of classifiers in https://pypi.python.org/pypi?:action=list_classifiers

                 ],
)"""


    #----------------------------------------------------------------------
    @classmethod
    def get_manifest_template(cls):
        return """
recursive-include p8 *
recursive-include p32 *
recursive-include pdl *
recursive-include examples *
recursive-exclude * __pycache__
recursive-exclude * *.py[co]"""




#!/usr/bin/env python
#-*- coding: utf-8 -*-

from PySide.QtGui import QWidget
from PySide import QtCore, QtGui

from ..code_editor.line_number import LineNumber
from ..code_editor.editor import CustomTextEdit

########################################################################
class PinguinoCodeEditor(QWidget):

    #----------------------------------------------------------------------
    def __init__(self, highlighter):

        super(PinguinoCodeEditor, self).__init__()

        self.setObjectName("PinguinoCodeEditor")

        self.gridLayout_0 = QtGui.QGridLayout(self)
        self.gridLayout_0.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_0.setSpacing(0)

        self.line_number = LineNumber(self)
        self.line_number.setMinimumSize(QtCore.QSize(41, 0))
        self.line_number.setMaximumSize(QtCore.QSize(41, 16777215))
        self.gridLayout_0.addWidget(self.line_number, 0, 0, 1, 1)

        self.text_edit = CustomTextEdit(self, self.line_number, highlighter)
        self.text_edit.setFrameShape(QtGui.QFrame.NoFrame)
        self.gridLayout_0.addWidget(self.text_edit, 0, 3, 1, 1)

        self.widget = QtGui.QWidget(self)
        self.widget.setMinimumSize(QtCore.QSize(10, 0))
        self.gridLayout_0.addWidget(self.widget, 0, 2, 1, 1)


        palette = QtGui.QPalette(self.widget.palette())
        self.widget.setAutoFillBackground(True)
        palette.setColor(QtGui.QPalette.Window, QtGui.QColor("#F0F0F0"))
        self.widget.setPalette(palette)

        #palette = QtGui.QPalette(self.palette())
        #self.setAutoFillBackground(True)
        #palette.setColor(QtGui.QPalette.Base, QtGui.QColor("#FFFFFF"))
        #self.setPalette(palette)

        self.line_number.setTextEdit(self.text_edit)
        self.text_edit.viewport().installEventFilter(self)


    #----------------------------------------------------------------------
    def set_autocompleter(self, autocompleter):
        """"""
        self.text_edit.set_autocompleter(autocompleter)

    #----------------------------------------------------------------------
    def eventFilter(self, object_, event):

        self.line_number.update()
        self.text_edit.brace_match()
        return False


    #----------------------------------------------------------------------
    def set_font_size(self, font_size):

        font = self.text_edit.font()
        font.setPointSize(font_size)
        self.text_edit.setFont(font)


    #----------------------------------------------------------------------
    def set_font_family(self, font_family, fixed_pitch=True):

        font = self.text_edit.font()
        font.setFamily(font_family)
        font.setFixedPitch = fixed_pitch
        self.text_edit.setFont(font)



























#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import logging
from datetime import datetime

from PySide import QtGui, QtCore
import requests
import json

# Python3 compatibility
if os.getenv("PINGUINO_PYTHON") is "3":
    #Python3
    #from urllib.request import urlopen
    #from urllib.parse import urlencode
    from configparser import RawConfigParser
else:
    #Python2
    #from urllib import urlopen, urlencode
    from ConfigParser import RawConfigParser

from ...frames.submit_bug import Ui_SubmitBug
from ..methods.dialogs import Dialogs

#if os.getenv("PINGUINO_MODE") == "NORMAL":
    #SUBMIT_SERVER = "http://submit.pinguino.xyz/submit/"
#else:
    #SUBMIT_SERVER = "http://localhost:8000/submit/"

########################################################################
class SubmitBug(QtGui.QDialog):

    def __init__(self, parent, details=""):
        super(SubmitBug, self).__init__()
        self.setWindowFlags(QtCore.Qt.WindowCloseButtonHint |
                            QtCore.Qt.WindowSystemMenuHint |
                            QtCore.Qt.WindowStaysOnTopHint |
                            QtCore.Qt.Tool)

        self.submit = Ui_SubmitBug()
        self.submit.setupUi(self)
        self.main = parent

        username, password = self.get_auth()
        self.submit.lineEdit_username.setText(username)
        self.submit.lineEdit_password.setText(password)

        self.submit.plainTextEdit_details.insertPlainText(details)

        self.setWindowTitle(os.getenv("PINGUINO_NAME")+" - "+self.windowTitle())

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/logo/art/windowIcon.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.setWindowIcon(icon)

        self.connect(self.submit.pushButton_submit, QtCore.SIGNAL("clicked()"), self.submit_now)
        self.connect(self.submit.pushButton_cancel, QtCore.SIGNAL("clicked()"), self.close)
        self.connect(self.submit.checkBox_show_this_dialog, QtCore.SIGNAL("clicked()"), self.update_submit_dialog)

        self.setStyleSheet("""
        font-family: inherit;
        font-weight: normal;

        """)

        self.center_on_screen()

    #----------------------------------------------------------------------
    def center_on_screen(self):

        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)


    #----------------------------------------------------------------------
    def update_submit_dialog(self):

        self.main.configIDE.set("Features", "submmit_bugs", self.submit.checkBox_show_this_dialog.isChecked())


    #----------------------------------------------------------------------
    def submit_now(self):
        """"""

        summary = self.submit.lineEdit_summary.text().replace("\n", "<br>")
        details = self.submit.plainTextEdit_details.toPlainText()
        username = self.submit.lineEdit_username.text()
        password = self.submit.lineEdit_password.text()
        repo = self.submit.comboBox_repo.currentText()

        if not details:
            Dialogs.error_message(self, "No details!")
            return

        if not username:
            Dialogs.error_message(self, "No userbane!")
            return

        if not password:
            Dialogs.error_message(self, "No password!")
            return

        environ = self.get_systeminfo()
        logging.info("Submitting bug report.")
        maked = self.make_github_issue(summary, details, repo, environ, username, password)
        #response = urlopen(SUBMIT_SERVER, urlencode({"summary": summary, "details": details, "environ": environ,}).encode("utf-8"))
        #share_link = eval(response.read())["share"]
        self.hide()

        if not maked:
            if details and environ:
                self.save_for_later(summary, details, repo, environ, username, password)
            logging.error("ConnectionError")

        self.close()


    #----------------------------------------------------------------------
    def get_systeminfo(self):

        data = {}
        try: data["os.name"] = str(os.name)
        except: pass
        try: data["os.environ"] = str(os.environ)
        except: pass
        try: data["os.uname"] = str(os.uname())
        except: pass
        try: data["sys.argv"] = str(sys.argv)
        except: pass
        try: data["sys.flags"] = str(sys.flags)
        except: pass
        try: data["sys.platform"] = str(sys.platform)
        except: pass
        try: data["sys.version"] = str(sys.version)
        except: pass
        try: data["platform.architecture"] = str(platform.architecture())
        except: pass
        try: data["platform.dist"] = str(platform.dist())
        except: pass
        try: data["platform.linux_distribution"] = str(platform.linux_distribution())
        except: pass
        try: data["platform.mac_ver"] = str(platform.mac_ver())
        except: pass
        try: data["platform.system"] = str(platform.system())
        except: pass
        try: data["platform.win32_ver"] = str(platform.win32_ver())
        except: pass
        try: data["platform.libc_ver"] = str(platform.libc_ver())
        except: pass
        try: data["platform.machine"] = str(platform.machine())
        except: pass
        try: data["platform.platform"] = str(platform.platform())
        except: pass
        try: data["platform.release"] = str(platform.release())
        except: pass
        try: data["pyusb"] = str(usb.__version__)
        except: pass

        return "\n".join([": ".join(item) for item in data.items()])


    #----------------------------------------------------------------------
    def save_for_later(self, summary, details, repo, environ, username, password):
        """"""
        parser = RawConfigParser()
        parser.add_section("SUBMIT")
        parser.set("SUBMIT", "summary", summary)
        parser.set("SUBMIT", "details", details)
        parser.set("SUBMIT", "repo", repo)
        parser.set("SUBMIT", "environ", environ)
        parser.set("SUBMIT", "username", username)
        parser.set("SUBMIT", "password", password)

        filename = os.path.join(os.getenv("PINGUINO_USER_PATH"), "submit-{}".format(datetime.now()))
        parser.write(open(filename, "w"))


    #----------------------------------------------------------------------
    def save_auth(self, username, password):
        """"""
        parser = RawConfigParser()
        parser.add_section("AUTH")
        parser.set("AUTH", "username", username)
        parser.set("AUTH", "password", password)

        filename = os.path.join(os.getenv("PINGUINO_USER_PATH"), "submit-auth")
        parser.write(open(filename, "w"))


    #----------------------------------------------------------------------
    def get_auth(self):
        """"""
        if not os.path.exists(os.path.join(os.getenv("PINGUINO_USER_PATH"), "submit-auth")):
            return '', ''

        parser = RawConfigParser()
        filename = os.path.join(os.getenv("PINGUINO_USER_PATH"), "submit-auth")
        parser.readfp(open(filename, "r"))

        username = parser.get("AUTH", "username")
        password = parser.get("AUTH", "password")

        return username, password


    #----------------------------------------------------------------------
    def make_github_issue(self, summary, details, repo, environ, username, password):
        '''Create an issue on github.com using the given parameters.'''
        # Our url to create issues via POST
        url = 'https://api.github.com/repos/{}/{}/issues'.format('PinguinoIDE', repo)
        # Create an authenticated session to create the issue
        #session = requests.session(auth=(username, password))
        session = requests.Session()
        session.auth = (username, password)

        # Create our issue
        issue = {'title': summary,
                 'body': "{}\n\n{}".format(details, environ),
                 'labels': ['submitted-from-ide',
                            'bug',
                            'v{}'.format(os.environ["PINGUINO_VERSION"][:2]),
                            ],
                 }

        # Add the issue to our repository
        r = session.post(url, json.dumps(issue))
        if r.status_code == 201:
            logging.info('Successfully created Issue "{}"'.format(summary))
            self.save_auth(username, password)

        return r.status_code == 201


#----------------------------------------------------------------------
def send_old_submits():

    submits = filter(lambda s:s.startswith("submit-"), os.listdir(os.getenv("PINGUINO_USER_PATH")))
    for submit in submits:
        parser = RawConfigParser()
        filename = os.path.join(os.getenv("PINGUINO_USER_PATH"), submit)
        parser.readfp(open(filename, "r"))

        summary = parser.get("SUBMIT", "summary")
        details = parser.get("SUBMIT", "details")
        repo = parser.get("SUBMIT", "repo")
        environ = parser.get("SUBMIT", "environ")
        username = parser.get("SUBMIT", "username")
        password = parser.get("SUBMIT", "password")


        try:
            url = 'https://api.github.com/repos/{}/{}/issues'.form
            session = requests.Session()
            session.auth = (username, password)
            issue = {'title': summary,
                     'body': "{}\n\n{}".format(details, environ),
                     'labels': ['submitted-from-ide',
                                'bug',
                                'v{}'.format(os.environ["PINGUINO_VERSION"][:2]),
                                ],
                     }
            r = session.post(url, json.dumps(issue))
            os.remove(filename)
        except:
            pass





#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
import re
import codecs
import pickle

from ..methods.decorators import Decorator

from ...pinguino_core.pinguino_config import PinguinoConfig

from PySide import QtCore, QtGui

# data_types = ["int", "float", "char", "double", "u8", "u16", "u32", "u64", "BOOL", "byte", "word", "void", "short", "long"]  # Yes I know, void is NOT a data type.
# preprocessor_commands = ["define", "include", "error", "undef", "if", "else", "if", "elif", "ifdef", "ifndef", "line", "pragma"]  #endif


########################################################################
class SourceBrowser(object):

    #----------------------------------------------------------------------
    def __init__(self):

        self.connect(self.main.tableWidget_functions, QtCore.SIGNAL("doubleClicked(QModelIndex)"), self.jump_function)
        self.connect(self.main.tableWidget_directives, QtCore.SIGNAL("doubleClicked(QModelIndex)"), self.jump_directive)
        self.connect(self.main.tableWidget_variables, QtCore.SIGNAL("doubleClicked(QModelIndex)"), self.jump_variable)
        self.connect(self.main.tableWidget_functions.verticalHeader(), QtCore.SIGNAL("sectionClicked(int)"), self.jump_function)
        self.connect(self.main.tableWidget_directives.verticalHeader(), QtCore.SIGNAL("sectionClicked(int)"), self.jump_directive)
        self.connect(self.main.tableWidget_variables.verticalHeader(), QtCore.SIGNAL("sectionClicked(int)"), self.jump_variable)

        self.connect(self.main.label_library_edit, QtCore.SIGNAL("linkActivated(QString)"), self.open_file_from_link)
        # self.connect(self.main.label_headers_edit, QtCore.SIGNAL("linkActivated(QString)"), self.open_file_from_link)


    #----------------------------------------------------------------------
    def set_functions(self, functions):

        self.functions = functions
        # self.update_functions()


    #----------------------------------------------------------------------
    def get_functions(self):

        if hasattr(self, "functions"): return self.functions
        else: return []


    #----------------------------------------------------------------------
    def set_directives(self, directives):

        self.directives = directives
        # self.update_directives()


    #----------------------------------------------------------------------
    def get_directives(self):

        if hasattr(self, "directives"): return self.directives
        else: return []


    #----------------------------------------------------------------------
    def set_variables(self, variables):

        self.variables = variables
        # self.update_variables()


    #----------------------------------------------------------------------
    def get_variables(self):

        if hasattr(self, "variables"): return self.variables
        else: return []


    #----------------------------------------------------------------------
    @Decorator.clear_highlighted_lines()
    def jump_function(self, row):

        if type(row) == int:
            item = self.main.tableWidget_functions.verticalHeaderItem(row)
        else:
            item = self.main.tableWidget_functions.verticalHeaderItem(row.column())

        if getattr(item, "filename", False):
            self.ide_open_file_from_path(filename=item.filename)
        self.editor_highligh_line(item.line, "#DBFFE3")


    #----------------------------------------------------------------------
    @Decorator.clear_highlighted_lines()
    def jump_variable(self, row):

        if type(row) == int:
            item = self.main.tableWidget_variables.verticalHeaderItem(row)
        else:
            item = self.main.tableWidget_variables.verticalHeaderItem(row.column())

        if getattr(item, "filename", False):
            self.ide_open_file_from_path(filename=item.filename)
        self.editor_highligh_line(item.line, "#DBFFE3")


    #----------------------------------------------------------------------
    @Decorator.clear_highlighted_lines()
    def jump_directive(self, row):

        if type(row) == int:
            item = self.main.tableWidget_directives.verticalHeaderItem(row)
        else:
            item = self.main.tableWidget_directives.verticalHeaderItem(row.column())

        if getattr(item, "filename", False):
            self.ide_open_file_from_path(filename=item.filename)
        self.editor_highligh_line(item.line, "#DBFFE3")


    #----------------------------------------------------------------------
    def get_files_to_explore(self):
        """"""
        if self.is_project() and not self.is_library():
            """"""
            # files = self.get_files_from_project()

            filenames = [s[0] for s in filter(lambda l:l[1], self.get_files_from_project().items())]
            filenames.extend([s[0] for s in filter(lambda l:l[1], self.get_inherits_from_project().items())])

            files = [{"filename": filename, "content": str(codecs.open(filename, encoding="utf-8").read()),} for filename in filenames]
            return files

        else:
            if self.is_graphical() is False:
                return [{"filename": "",
                         "content": str(self.get_current_editor().text_edit.toPlainText())}]
            else:
                return []



    #----------------------------------------------------------------------
    def update_assistant(self, name):
        """"""
        if not hasattr(self, "assistant"): self.set_assistant()

        if name in self.assistant:

            link = "<a href='{text}'><span style='text-decoration: none; color:#2980b9;'>{text}</span></a>"
            html = "<html><head/><body><p>{links}</p></body></html>"

            self.main.label_name_edit.setText("{} ({name})".format(name, **self.assistant[name]))
            self.main.label_args_edit.setText("({args})".format(**self.assistant[name]))
            self.main.label_library_edit.setText(html.format(links=link.format(text=self.assistant[name]["filename"])))

            return_ = self.assistant[name]["return"]
            if return_ == "void": return_ = "None"
            self.main.label_return_edit.setText("{}".format(return_))

            # return True
            self.main.label_source_browser.setVisible(False)


    #----------------------------------------------------------------------
    def set_assistant(self):
        """"""
        with open(os.path.join(os.getenv("PINGUINO_USER_PATH"), "reserved.pickle"), "rb") as file_reserved:
            self.assistant = pickle.load(file_reserved).get("assistant", {})


    #----------------------------------------------------------------------
    def open_file_from_link(self, name):
        """"""
        core = os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "p8", "include", "pinguino", "core")
        libraries = os.path.join(os.getenv("PINGUINO_INSTALL_PATH"), "p8", "include", "pinguino", "libraries")

        path = None
        if name in os.listdir(core):
            path =  os.path.join(core, name)
        elif name in os.listdir(libraries):
            path =  os.path.join(libraries, name)

        if path is None:
            for path_ in PinguinoConfig.get_p8_libraries() + PinguinoConfig.get_p32_libraries():
                if name in os.listdir(path_):
                    path = os.path.join(path_, name)
                    break

        if os.path.exists(path):
            self.ide_open_file_from_path(filename=path, readonly=True)



    # #----------------------------------------------------------------------
    # def update_code_navigator(self):
        # """"""
        # self.update_variables()
        # self.update_functions()
        # self.update_directives()




#!/usr/bin/env python
#-*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages

if os.name in ["posix", "os2"]: #GNU/Linux and MacOS
    os_kwargs = {"data_files": [
                                ("/usr/share/applications", ["pinguino12.desktop"]),
                                ("/usr/share/icons/hicolor/256x256", ["pinguino/qtgui/resources/art/pinguino_logo-256x256.png"]),
                                ("/usr/share/icons/hicolor/256x256", ["pinguino/qtgui/resources/art/pinguino_logo_background_blue-256x256.png"]),
                                ],
                }

# elif os.name == "os2":  #Mac OS X
    # os_kwargs = {}

elif os.name == "nt":  #Windows
    os_kwargs = {"data_files": [
                                # ("C:\pinguino", ["pinguino12.desktop"]),
                                ("C:\pinguino\icons", ["pinguino/qtgui/resources/art/pinguino_logo-256x256.ico"]),
                                ("C:\pinguino\icons", ["pinguino/qtgui/resources/art/pinguino_logo_background_blue-256x256.ico"]),
                                ],
                }

from pinguino import version

setup(name = "pinguino",
      version = "{}.{}.{}".format(version.MAJOR, version.MINOR, version.PATCH),
      description = "Open Hardware Electronics Prototyping Platform, Open Source Integrated Development Environment (IDE)",
      description_file = "README.rst",

      author = "",
      author_email = "",
      maintainer = "Yeison Cardona",
      maintainer_email = "yeisoneng@gmail.com",

      url = "http://www.pinguino.cc/",
      download_url = "",

      license = "GPLv2",
      keywords = "microchip, electronic, prototyping, IDE",

      classifiers=[#list of classifiers in https://pypi.python.org/pypi?:action=list_classifiers
                   "Environment :: X11 Applications :: Qt",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
                   "Operating System :: POSIX :: Linux",
                   "Operating System :: Microsoft :: Windows",
                   "Operating System :: MacOS",
                   "Programming Language :: Python",
                   "Topic :: Software Development",
                   "Topic :: Software Development :: Code Generators",
                   "Topic :: Software Development :: Compilers",
                   "Topic :: Software Development :: Debuggers",
                   ],

      packages = find_packages(),
      include_package_data = True,

      install_requires = [
                          #"gitpython",
                          #"hgapi",
                          "pyusb==1.0.0b2",
                          #"setuptools",
                          #"wheel"
                          #"pyside",
                          ],

      extras_require = {
          "gui":  ["pyside", 'requests']
      },

      scripts = [
                 "cmd/pinguinoide",			# pinguino ide launcher
                 "cmd/pinguinoide.pyw",		# pinguino ide launcher with extension for windows
                 "cmd/pinguino-cmd",		# pinguino command line
                 "cmd/pinguino-cmd.py",		# pinguino command line with extension for windows
                 "cmd/pinguino-reset",
                 "cmd/pinguino-reset.py",
                 "cmd/pinguino-test",
                 "cmd/pinguino-test.py",
                 ],

      zip_safe = False,

      **os_kwargs
      )
